# coding: utf-8

from .abstract_config import config_root, generate_respath
from .indexurl_manager import IndexManagerConfig
from .main_entrance import MainEntranceConfig
from .package_download import PackageDownloadConfig
from .package_manager import PackageManagerConfig
from .pyinstaller_tool import PyiConfigure, PyinstallerToolConfig

__all__ = [
    "config_root",
    "generate_respath",
    "IndexManagerConfig",
    "MainEntranceConfig",
    "PackageDownloadConfig",
    "PackageManagerConfig",
    "PyiConfigure",
    "PyinstallerToolConfig",
]



# coding: utf-8

import os

from chardet import detect
from PyQt5.QtWidgets import *

from .messagebox import MessageBox


class QueryFilePath(QWidget):
    def load_from_text(self, last_path):
        text_file_path, _ = QFileDialog.getOpenFileName(
            self, "选择文本文件", last_path, "文本文件 (*.txt)"
        )
        if not text_file_path:
            return "", ""
        try:
            with open(text_file_path, "rb") as f:
                encoding = detect(f.read()).get("encoding", "utf-8")
            with open(text_file_path, "rt", encoding=encoding) as obj:
                return obj.read(), os.path.dirname(text_file_path)
        except Exception as reason:
            MessageBox(
                "错误",
                f"文件打开失败：\n{str(reason)}",
                QMessageBox.Critical,
            ).exec_()
            return "", ""

    def save_as_text_file(self, data, last_path):
        save_path, _ = QFileDialog.getSaveFileName(
            self, "保存文件", last_path, "文本文件 (*.txt)"
        )
        if not save_path:
            return ""
        try:
            with open(save_path, "wt", encoding="utf-8") as fobj:
                fobj.writelines(data)
        except Exception as reason:
            return MessageBox(
                "错误", f"文件保存失败：\n{str(reason)}", QMessageBox.Critical
            ).exec_()
        return os.path.dirname(save_path)

    def get_dir_path(self, last_path):
        _path = QFileDialog.getExistingDirectory(self, "选择目录", last_path)
        if _path:
            return os.path.normpath(_path)
        return ""



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'index_manager.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_index_manager(object):
    def setupUi(self, index_manager):
        index_manager.setObjectName("index_manager")
        index_manager.setWindowModality(QtCore.Qt.WindowModal)
        index_manager.resize(480, 520)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        index_manager.setFont(font)
        self.centralwidget = QtWidgets.QWidget(index_manager)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.li_indexurls = QtWidgets.QListWidget(self.centralwidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(155, 222, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(155, 222, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(190, 190, 190))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        self.li_indexurls.setPalette(palette)
        self.li_indexurls.setObjectName("li_indexurls")
        self.verticalLayout_4.addWidget(self.li_indexurls)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setText("")
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.btn_delurl = QtWidgets.QPushButton(self.centralwidget)
        self.btn_delurl.setObjectName("btn_delurl")
        self.horizontalLayout_4.addWidget(self.btn_delurl)
        self.horizontalLayout_4.setStretch(0, 9)
        self.horizontalLayout_4.setStretch(1, 1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_4.addWidget(self.line)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.le_urlname = QtWidgets.QLineEdit(self.centralwidget)
        self.le_urlname.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_urlname.setObjectName("le_urlname")
        self.verticalLayout.addWidget(self.le_urlname)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.le_indexurl = QtWidgets.QLineEdit(self.centralwidget)
        self.le_indexurl.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_indexurl.setObjectName("le_indexurl")
        self.verticalLayout_2.addWidget(self.le_indexurl)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.horizontalLayout_2.setStretch(0, 2)
        self.horizontalLayout_2.setStretch(1, 9)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_5 = QtWidgets.QLabel(self.centralwidget)
        self.label_5.setText("")
        self.label_5.setObjectName("label_5")
        self.horizontalLayout.addWidget(self.label_5)
        self.btn_clearle = QtWidgets.QPushButton(self.centralwidget)
        self.btn_clearle.setObjectName("btn_clearle")
        self.horizontalLayout.addWidget(self.btn_clearle)
        self.btn_saveurl = QtWidgets.QPushButton(self.centralwidget)
        self.btn_saveurl.setObjectName("btn_saveurl")
        self.horizontalLayout.addWidget(self.btn_saveurl)
        self.btn_setindex = QtWidgets.QPushButton(self.centralwidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(120, 120, 120))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
        self.btn_setindex.setPalette(palette)
        self.btn_setindex.setObjectName("btn_setindex")
        self.horizontalLayout.addWidget(self.btn_setindex)
        self.horizontalLayout.setStretch(0, 9)
        self.horizontalLayout.setStretch(1, 2)
        self.horizontalLayout.setStretch(2, 2)
        self.horizontalLayout.setStretch(3, 2)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.line_2 = QtWidgets.QFrame(self.centralwidget)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_4.addWidget(self.line_2)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.btn_refresh_effective = QtWidgets.QPushButton(self.centralwidget)
        self.btn_refresh_effective.setObjectName("btn_refresh_effective")
        self.horizontalLayout_3.addWidget(self.btn_refresh_effective)
        self.horizontalLayout_3.setStretch(0, 9)
        self.horizontalLayout_3.setStretch(1, 1)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.le_effectiveurl = QtWidgets.QLineEdit(self.centralwidget)
        self.le_effectiveurl.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_effectiveurl.setReadOnly(True)
        self.le_effectiveurl.setObjectName("le_effectiveurl")
        self.verticalLayout_3.addWidget(self.le_effectiveurl)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.verticalLayout_4.setStretch(0, 9)
        self.verticalLayout_4.setStretch(1, 1)
        self.verticalLayout_4.setStretch(2, 1)
        self.verticalLayout_4.setStretch(3, 1)
        self.verticalLayout_4.setStretch(4, 1)
        self.verticalLayout_4.setStretch(5, 1)
        self.verticalLayout_4.setStretch(6, 1)
        self.verticalLayout_5.addLayout(self.verticalLayout_4)
        index_manager.setCentralWidget(self.centralwidget)

        self.retranslateUi(index_manager)
        QtCore.QMetaObject.connectSlotsByName(index_manager)

    def retranslateUi(self, index_manager):
        _translate = QtCore.QCoreApplication.translate
        index_manager.setWindowTitle(_translate("index_manager", "镜像源设置"))
        self.btn_delurl.setToolTip(_translate("index_manager", "删除列表中选中的镜像源地址。"))
        self.btn_delurl.setText(_translate("index_manager", "删除"))
        self.label.setText(_translate("index_manager", "编辑名称："))
        self.label_2.setText(_translate("index_manager", "编辑地址："))
        self.btn_clearle.setToolTip(_translate("index_manager", "清空正在编辑的名称和地址输入框。"))
        self.btn_clearle.setText(_translate("index_manager", "清空编辑"))
        self.btn_saveurl.setToolTip(_translate("index_manager", "将输入框内的名称和地址保存到列表。"))
        self.btn_saveurl.setText(_translate("index_manager", "保存至列表"))
        self.btn_setindex.setToolTip(_translate("index_manager", "将输入框内的镜像源地址设置为全局镜像源。"))
        self.btn_setindex.setText(_translate("index_manager", "启用镜像源"))
        self.label_3.setText(_translate("index_manager", "当前生效的全局镜像源地址："))
        self.btn_refresh_effective.setToolTip(_translate("index_manager", "获取当前生效的全局镜像源地址并显示。"))
        self.btn_refresh_effective.setText(_translate("index_manager", "刷新"))



# coding: utf-8

from PyQt5.QtWidgets import *


class MessageBox(QMessageBox):
    """
    点击按钮后返回该按钮在参数中的次序值

    多按钮：关闭窗口返回值跟随'reject'按钮次序值

    只有一个按钮：直接关闭窗口返回 0

    多按钮的情况下，'reject'按钮在最右侧

    有'destructive'按钮，无'reject'按钮，窗口不可关闭
    """

    def __init__(
        self, title, message, icon=None, buttons=(("accept", "确定"),)
    ):
        if icon is None:
            icon = QMessageBox.Information
        super().__init__(icon, title, message)
        self._buttons = buttons
        self.__set_push_buttons()

    def __set_push_buttons(self):
        for btn in self._buttons:
            role, text = btn
            if role == "accept":
                self.addButton(text, QMessageBox.AcceptRole)
            elif role == "destructive":
                self.addButton(text, QMessageBox.DestructiveRole)
            elif role == "reject":
                self.setDefaultButton(self.addButton(text, QMessageBox.RejectRole))

    def get_role(self):
        return self.exec_()



# coding: utf-8

__doc__ = """包含AwesomePyKit的主要类、函数、配置文件路径等。"""

import os.path as op
import re
import subprocess
from subprocess import *

from fastpip import PyEnv
from PyQt5.QtCore import *


def loop_install(
    pyenv, sequence, *, index_url="", pre=False, user=False, upgrade=False
):
    for name in sequence:
        cmd_exec_result = pyenv.install(
            name,
            pre=pre,
            user=user,
            index_url=index_url,
            upgrade=upgrade,
        )
        yield cmd_exec_result[0][0], cmd_exec_result[1]


def loop_uninstall(pyenv, sequence):
    for name in sequence:
        cmd_exec_result = pyenv.uninstall(name)
        yield cmd_exec_result[0][0], cmd_exec_result[1]


def check_py_path(py_dir_path):
    return PyEnv(py_dir_path).env_path != ""


def clean_py_paths(paths):
    return [pth for pth in paths if check_py_path(pth)]


def check_index_url(url):
    return bool(re.match(r"^https://.+/simple[/]?$", url.lower()))


def clean_index_urls(urls):
    return [url for url in urls if check_index_url(url)]


class QThreadModel(QThread):
    def __init__(self, target, args=tuple()):
        super().__init__()
        self._args = args
        self._target = target
        self._at_start = list()
        self._at_finish = list()

    def run(self):
        self._target(*self._args)

    def __repr__(self):
        return f"{self._target} with args:{self._args}"

    __str__ = __repr__

    def at_start(self, *callable_objs):
        for cab in callable_objs:
            self.started.connect(cab)
        self._at_start.extend(callable_objs)

    def at_finish(self, *callable_objs):
        for cab in callable_objs:
            self.finished.connect(cab)
        self._at_finish.extend(callable_objs)

    def broken_signal(self):
        for cab in self._at_start:
            self.started.disconnect(cab)
        for cab in self._at_finish:
            self.finished.disconnect(cab)


class ThreadRepo:
    def __init__(self, interval):
        """interval: 清理已结束线程的时间间隔，单位毫秒。"""
        self._thread_repo = []
        self._timer_clths = QTimer()
        self._mutex = QMutex()
        self._timer_clths.timeout.connect(self.clean)
        self._timer_clths.start(interval)
        self._flag_cleaning = False

    def put(self, threadhandle, level=0):
        """将(线程句柄、重要等级)元组加入线程仓库。"""
        self._mutex.lock()
        self._thread_repo.append((threadhandle, level))
        self._mutex.unlock()

    def clean(self):
        """清除已结束的线程。"""
        if self._flag_cleaning:
            return
        self._mutex.lock()
        index = 0
        self._flag_cleaning = True
        while index < len(self._thread_repo):
            if self._thread_repo[index][0].isRunning():
                index += 1
                continue
            del self._thread_repo[index]
        self._flag_cleaning = False
        self._mutex.unlock()

    def stop_all(self):
        """
        按线程重要等级退出线程。
        0级：重要，安全退出；
        1级：不重要，立即退出；
        其他：未知等级，安全退出。
        """
        for thread, level in self._thread_repo:
            thread.broken_signal()
            if level == 0:
                thread.quit()
            elif level == 1:
                thread.terminate()
            else:
                thread.quit()

    def kill_all(self):
        """立即终止所有线程。"""
        for thread, _ in self._thread_repo:
            thread.broken_signal()
            thread.terminate()

    def is_empty(self):
        """返回线程仓库是否为空。"""
        return not self._thread_repo


def get_cmd_out(*commands, regexp="", timeout=None):
    """用于从cmd命令执行输出的字符匹配想要的信息。"""
    info = STARTUPINFO()
    info.dwFlags = STARTF_USESHOWWINDOW
    info.wShowWindow = SW_HIDE
    proc = Popen(commands, stdout=PIPE, text=True, startupinfo=info)
    try:
        strings, _ = proc.communicate(timeout=timeout)
    except Exception:
        return ""
    if not regexp:
        return strings.strip()
    return re.search(regexp, strings)


def open_explorer(fd_path, option="root"):
    """
    option 参数：

    为 root 表示以 fd_path 为根目录打开资源管理器

    为 select 表示打开 fd_path 上一级目录并选中 fd_path 所指文件或目录
    """
    assert isinstance(fd_path, str)
    assert option in ("root", "select")
    fd_path = op.normpath(fd_path.strip())
    if op.isfile(fd_path):
        commands = f"explorer /select,{fd_path}"
    elif op.isdir(fd_path):
        commands = f"explorer /{option},{fd_path}"
    else:
        return
    subprocess.run(commands)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'package_install.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_package_install(object):
    def setupUi(self, package_install):
        package_install.setObjectName("package_install")
        package_install.setWindowModality(QtCore.Qt.WindowModal)
        package_install.resize(393, 416)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        package_install.setFont(font)
        package_install.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.centralwidget = QtWidgets.QWidget(package_install)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.uiLabel_target_environment = QtWidgets.QLabel(self.centralwidget)
        self.uiLabel_target_environment.setFrameShape(QtWidgets.QFrame.Box)
        self.uiLabel_target_environment.setObjectName("uiLabel_target_environment")
        self.horizontalLayout.addWidget(self.uiLabel_target_environment)
        self.horizontalLayout.setStretch(1, 1)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.line_3 = QtWidgets.QFrame(self.centralwidget)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_4.addWidget(self.line_3)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setSpacing(2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.uiHorizontalLayout_package_name = QtWidgets.QHBoxLayout()
        self.uiHorizontalLayout_package_name.setSpacing(8)
        self.uiHorizontalLayout_package_name.setObjectName("uiHorizontalLayout_package_name")
        self.pte_package_names_old = QtWidgets.QPlainTextEdit(self.centralwidget)
        self.pte_package_names_old.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.pte_package_names_old.setObjectName("pte_package_names_old")
        self.uiHorizontalLayout_package_name.addWidget(self.pte_package_names_old)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.cb_including_pre = QtWidgets.QCheckBox(self.centralwidget)
        self.cb_including_pre.setObjectName("cb_including_pre")
        self.verticalLayout_2.addWidget(self.cb_including_pre)
        self.cb_install_for_user = QtWidgets.QCheckBox(self.centralwidget)
        self.cb_install_for_user.setObjectName("cb_install_for_user")
        self.verticalLayout_2.addWidget(self.cb_install_for_user)
        self.line_2 = QtWidgets.QFrame(self.centralwidget)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_2.addWidget(self.line_2)
        self.pb_load_from_text = QtWidgets.QPushButton(self.centralwidget)
        self.pb_load_from_text.setObjectName("pb_load_from_text")
        self.verticalLayout_2.addWidget(self.pb_load_from_text)
        self.pb_save_as_text = QtWidgets.QPushButton(self.centralwidget)
        self.pb_save_as_text.setObjectName("pb_save_as_text")
        self.verticalLayout_2.addWidget(self.pb_save_as_text)
        self.pb_do_install = QtWidgets.QPushButton(self.centralwidget)
        self.pb_do_install.setMinimumSize(QtCore.QSize(0, 50))
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(120, 120, 120))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
        self.pb_do_install.setPalette(palette)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(12)
        self.pb_do_install.setFont(font)
        self.pb_do_install.setObjectName("pb_do_install")
        self.verticalLayout_2.addWidget(self.pb_do_install)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.verticalLayout_2.setStretch(6, 1)
        self.uiHorizontalLayout_package_name.addLayout(self.verticalLayout_2)
        self.uiHorizontalLayout_package_name.setStretch(0, 1)
        self.verticalLayout_3.addLayout(self.uiHorizontalLayout_package_name)
        self.verticalLayout_3.setStretch(1, 1)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.line_4 = QtWidgets.QFrame(self.centralwidget)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_4.addWidget(self.line_4)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.cb_use_index_url = QtWidgets.QCheckBox(self.centralwidget)
        self.cb_use_index_url.setObjectName("cb_use_index_url")
        self.verticalLayout.addWidget(self.cb_use_index_url)
        self.le_use_index_url = QtWidgets.QLineEdit(self.centralwidget)
        self.le_use_index_url.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_use_index_url.setObjectName("le_use_index_url")
        self.verticalLayout.addWidget(self.le_use_index_url)
        self.verticalLayout_4.addLayout(self.verticalLayout)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_4.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(8)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_4.addWidget(self.label_2)
        package_install.setCentralWidget(self.centralwidget)

        self.retranslateUi(package_install)
        QtCore.QMetaObject.connectSlotsByName(package_install)

    def retranslateUi(self, package_install):
        _translate = QtCore.QCoreApplication.translate
        package_install.setWindowTitle(_translate("package_install", "安装"))
        self.label_3.setToolTip(_translate("package_install", "当前即将安装的包的目标环境。"))
        self.label_3.setText(_translate("package_install", "目标环境："))
        self.label.setText(_translate("package_install", "要安装的包名称（每行一个）："))
        self.cb_including_pre.setToolTip(_translate("package_install", "从网络安装时是否查找包括预发行版和开发版在内的版本。\n"
"如果预发行版或开发版是最新版本，则安装预发行版或开发版。"))
        self.cb_including_pre.setText(_translate("package_install", "包括预发行版和开发版"))
        self.cb_install_for_user.setToolTip(_translate("package_install", "将包安装到系统当前登录的用户的用户目录内。"))
        self.cb_install_for_user.setText(_translate("package_install", "仅为系统当前用户安装"))
        self.pb_load_from_text.setToolTip(_translate("package_install", "从文本文件载入名称及版本等内容。\n"
"例如从常见的requirements.txt文件载入。"))
        self.pb_load_from_text.setText(_translate("package_install", "从文件加载名称"))
        self.pb_save_as_text.setToolTip(_translate("package_install", "将文本框内的内容保存至文本文件。"))
        self.pb_save_as_text.setText(_translate("package_install", "名称保存至文件"))
        self.pb_do_install.setText(_translate("package_install", "开始安装"))
        self.cb_use_index_url.setText(_translate("package_install", "临时使用其他镜像源："))
        self.label_2.setText(_translate("package_install", "名称后支持跟随以下符号限定要安装的版本：\n"
"\"==\"、\">=\"、\"<=\"、\">\"、\"<\"、\",\"\n"
"每行一个名称，名称和限定符中不允许出现空格。\n"
"例如：fastpip>=0.6.2,<0.10.0\n"
"支持将whl文件拖入文本框内以从本地文件安装该包，whl文件需为正确版本。"))



# coding: utf-8

from .enums import Accept, AppStyle, Linkage, QMode
from .mapping import import_name
from .requires import REQ_FPVER

__all__ = ["Accept", "AppStyle", "import_name", "Linkage", "QMode", "REQ_FPVER"]



# coding: utf-8

__doc__ = "模块的 import 使用名与模块的发布名对照表"

import_name = {
    "PIL": "Pillow",
    "PyISAPI_loader": "pywin32",
    "adodbapi": "pywin32",
    "adsi": "pywin32",
    "afxres": "pywin32",
    "authorization": "pywin32",
    "axcontrol": "pywin32",
    "axdebug": "pywin32",
    "axscript": "pywin32",
    "bits": "pywin32",
    "bs4": "beautifulsoup4",
    "commctrl": "pywin32",
    "cv2": "opencv-python",
    "dbi": "pywin32",
    "dde": "pywin32",
    "directsound": "pywin32",
    "exchange": "pywin32",
    "exchdapi": "pywin32",
    "ifilter": "pywin32",
    "importlib_metadata": "importlib-metadata",
    "internet": "pywin32",
    "isapi": "pywin32",
    "mapi": "pywin32",
    "mmapfile": "pywin32",
    "mmsystem": "pywin32",
    "netbios": "pywin32",
    "ntsecuritycon": "pywin32",
    "odbc": "pywin32",
    "perfmon": "pywin32",
    "perfmondata": "pywin32",
    "pkg_resources": "setuptools",
    "propsys": "pywin32",
    "pythoncom": "pywin32",
    "pythonwin": "pywin32",
    "pywin32_bootstrap": "pywin32",
    "pywin32_testutil": "pywin32",
    "pywintypes": "pywin32",
    "rasutil": "pywin32",
    "regcheck": "pywin32",
    "regutil": "pywin32",
    "servicemanager": "pywin32",
    "shell": "pywin32",
    "sspi": "pywin32",
    "sspicon": "pywin32",
    "taskscheduler": "pywin32",
    "timer": "pywin32",
    "win2kras": "pywin32",
    "win32api": "pywin32",
    "win32clipboard": "pywin32",
    "win32com": "pywin32",
    "win32comext": "pywin32",
    "win32con": "pywin32",
    "win32console": "pywin32",
    "win32cred": "pywin32",
    "win32crypt": "pywin32",
    "win32cryptcon": "pywin32",
    "win32ctypes": "pywin32-ctypes",
    "win32event": "pywin32",
    "win32evtlog": "pywin32",
    "win32evtlogutil": "pywin32",
    "win32file": "pywin32",
    "win32gui": "pywin32",
    "win32gui_struct": "pywin32",
    "win32help": "pywin32",
    "win32inet": "pywin32",
    "win32inetcon": "pywin32",
    "win32job": "pywin32",
    "win32lz": "pywin32",
    "win32net": "pywin32",
    "win32netcon": "pywin32",
    "win32pdh": "pywin32",
    "win32pdhquery": "pywin32",
    "win32pdhutil": "pywin32",
    "win32pipe": "pywin32",
    "win32print": "pywin32",
    "win32process": "pywin32",
    "win32profile": "pywin32",
    "win32ras": "pywin32",
    "win32rcparser": "pywin32",
    "win32security": "pywin32",
    "win32service": "pywin32",
    "win32serviceutil": "pywin32",
    "win32timezone": "pywin32",
    "win32trace": "pywin32",
    "win32traceutil": "pywin32",
    "win32transaction": "pywin32",
    "win32ts": "pywin32",
    "win32ui": "pywin32",
    "win32uiole": "pywin32",
    "win32verstamp": "pywin32",
    "win32wnet": "pywin32",
    "winerror": "pywin32",
    "winioctlcon": "pywin32",
    "winnt": "pywin32",
    "winperf": "pywin32",
    "winxpgui": "pywin32",
    "winxptheme": "pywin32",
}



# coding: utf-8

from .indexurl_manager import IndexUrlManagerWindow
from .messagebox import MessageBox
from .package_download import PackageDownloadWindow
from .package_manager import PackageManagerWindow
from .pyinstaller_tool import PyinstallerToolWindow

__all__ = [
    "IndexUrlManagerWindow",
    "MessageBox",
    "PackageDownloadWindow",
    "PackageManagerWindow",
    "PyinstallerToolWindow",
]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'addenv_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_input_dialog(object):
    def setupUi(self, input_dialog):
        input_dialog.setObjectName("input_dialog")
        input_dialog.setWindowModality(QtCore.Qt.WindowModal)
        input_dialog.resize(580, 74)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        input_dialog.setFont(font)
        self.centralwidget = QtWidgets.QWidget(input_dialog)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(8, 16, 8, 8)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiLineEdit_input_content = QtWidgets.QLineEdit(self.centralwidget)
        self.uiLineEdit_input_content.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiLineEdit_input_content.setObjectName("uiLineEdit_input_content")
        self.horizontalLayout.addWidget(self.uiLineEdit_input_content)
        self.uiPushButton_select_envdir = QtWidgets.QPushButton(self.centralwidget)
        self.uiPushButton_select_envdir.setObjectName("uiPushButton_select_envdir")
        self.horizontalLayout.addWidget(self.uiPushButton_select_envdir)
        self.uiPushButton_confirm = QtWidgets.QPushButton(self.centralwidget)
        self.uiPushButton_confirm.setObjectName("uiPushButton_confirm")
        self.horizontalLayout.addWidget(self.uiPushButton_confirm)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.label = QtWidgets.QLabel(self.centralwidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(100, 100, 100))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(100, 100, 100))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(120, 120, 120))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        self.label.setPalette(palette)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(9)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.verticalLayout.setStretch(1, 1)
        input_dialog.setCentralWidget(self.centralwidget)

        self.retranslateUi(input_dialog)
        QtCore.QMetaObject.connectSlotsByName(input_dialog)

    def retranslateUi(self, input_dialog):
        _translate = QtCore.QCoreApplication.translate
        input_dialog.setWindowTitle(_translate("input_dialog", "输入内容"))
        self.uiPushButton_select_envdir.setToolTip(_translate("input_dialog", "打开目录选择窗口，选择 python.exe 文件所在的目录。"))
        self.uiPushButton_select_envdir.setText(_translate("input_dialog", "选择"))
        self.uiPushButton_confirm.setText(_translate("input_dialog", "确定"))
        self.label.setText(_translate("input_dialog", "粘贴 “python.exe” 文件所在目录路径或点击“选择”按钮选择 “python.exe” 文件所在目录然后点击确定。"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'imports_check.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_imports_check(object):
    def setupUi(self, imports_check):
        imports_check.setObjectName("imports_check")
        imports_check.setWindowModality(QtCore.Qt.WindowModal)
        imports_check.resize(900, 500)
        self.centralwidget = QtWidgets.QWidget(imports_check)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        self.centralwidget.setFont(font)
        self.centralwidget.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.le_cip_cur_env = QtWidgets.QLineEdit(self.centralwidget)
        self.le_cip_cur_env.setFrame(False)
        self.le_cip_cur_env.setReadOnly(True)
        self.le_cip_cur_env.setObjectName("le_cip_cur_env")
        self.horizontalLayout_2.addWidget(self.le_cip_cur_env)
        self.horizontalLayout_2.setStretch(1, 9)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tw_missing_imports = QtWidgets.QTableWidget(self.centralwidget)
        self.tw_missing_imports.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tw_missing_imports.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.tw_missing_imports.setObjectName("tw_missing_imports")
        self.tw_missing_imports.setColumnCount(3)
        self.tw_missing_imports.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tw_missing_imports.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_missing_imports.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_missing_imports.setHorizontalHeaderItem(2, item)
        self.tw_missing_imports.horizontalHeader().setHighlightSections(False)
        self.tw_missing_imports.verticalHeader().setHighlightSections(False)
        self.verticalLayout.addWidget(self.tw_missing_imports)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pb_install_all_missing = QtWidgets.QPushButton(self.centralwidget)
        self.pb_install_all_missing.setObjectName("pb_install_all_missing")
        self.horizontalLayout.addWidget(self.pb_install_all_missing)
        self.pb_confirm = QtWidgets.QPushButton(self.centralwidget)
        self.pb_confirm.setObjectName("pb_confirm")
        self.horizontalLayout.addWidget(self.pb_confirm)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        imports_check.setCentralWidget(self.centralwidget)

        self.retranslateUi(imports_check)
        QtCore.QMetaObject.connectSlotsByName(imports_check)

    def retranslateUi(self, imports_check):
        _translate = QtCore.QCoreApplication.translate
        imports_check.setWindowTitle(_translate("imports_check", "环境检查"))
        self.label.setText(_translate("imports_check", "当前环境:"))
        item = self.tw_missing_imports.horizontalHeaderItem(0)
        item.setText(_translate("imports_check", "项目文件"))
        item = self.tw_missing_imports.horizontalHeaderItem(1)
        item.setText(_translate("imports_check", "文件导入项"))
        item = self.tw_missing_imports.horizontalHeaderItem(2)
        item.setText(_translate("imports_check", "环境缺失项"))
        self.pb_install_all_missing.setToolTip(_translate("imports_check", "一键安装该环境所有缺失的项目导入项。\n"
"注意，当缺失的导入项无法在该环境安装时，不会有错误提示。"))
        self.pb_install_all_missing.setText(_translate("imports_check", "一键安装"))
        self.pb_confirm.setToolTip(_translate("imports_check", "什么都不做，关闭窗口并返回。"))
        self.pb_confirm.setText(_translate("imports_check", "确定"))



# coding: utf-8

from typing import Iterable, Sequence

from com import *

from .abstract_config import AbstractConfig

_shared_saved_pypaths = None


def get_shared_pypaths():
    return _shared_saved_pypaths


class PackageManagerConfig(AbstractConfig):
    """包管理器的设置类"""

    _key_python_paths = "python_paths"
    _key_include_pre = "include_pre"
    _key_install_for_user = "install_for_user"
    _key_index_url = "index_url"
    _key_last_path = "last_path"
    _key_use_index_url = "use_index_url"
    _key_package_names = "package_names"
    _key_window_size = "window_size"
    _key_install_winsize = "install_winsize"
    _key_input_winsize = "input_winsize"
    _key_output_side = "output_side"
    _key_output_winsize = "output_winsize"
    _key_query_winsize = "query_winsize"
    _key_query_name = "query_name"
    _key_query_mode = "query_mode"
    _key_query_case = "query_case"

    CONFIGFILE = "package_manager.json"

    def __init__(self):
        super().__init__(self.CONFIGFILE)
        global _shared_saved_pypaths
        _shared_saved_pypaths = self.pypaths

    @property
    def pypaths(self) -> list:
        if self._key_python_paths not in self:
            self[self._key_python_paths] = list()
        return self[self._key_python_paths]

    @pypaths.setter
    def pypaths(self, value: Iterable):
        assert isinstance(value, Iterable)
        assert all(isinstance(i, str) for i in value)
        self[self._key_python_paths].clear()
        self[self._key_python_paths].extend(value)

    @property
    def include_pre(self):
        if self._key_include_pre not in self:
            self[self._key_include_pre] = False
        return self[self._key_include_pre]

    @include_pre.setter
    def include_pre(self, value: bool):
        assert isinstance(value, bool)
        self[self._key_include_pre] = value

    @property
    def install_for_user(self):
        if self._key_install_for_user not in self:
            self[self._key_install_for_user] = False
        return self[self._key_install_for_user]

    @install_for_user.setter
    def install_for_user(self, value: bool):
        assert isinstance(value, bool)
        self[self._key_install_for_user] = value

    @property
    def last_path(self):
        if self._key_last_path not in self:
            self[self._key_last_path] = "."
        return self[self._key_last_path]

    @last_path.setter
    def last_path(self, value):
        assert isinstance(value, str)
        self[self._key_last_path] = value

    @property
    def index_url(self):
        if self._key_index_url not in self:
            self[self._key_index_url] = ""
        return self[self._key_index_url]

    @index_url.setter
    def index_url(self, value):
        assert isinstance(value, str)
        self[self._key_index_url] = value

    @property
    def use_index_url(self):
        if self._key_use_index_url not in self:
            self[self._key_use_index_url] = False
        return self[self._key_use_index_url]

    @use_index_url.setter
    def use_index_url(self, value: bool):
        assert isinstance(value, bool)
        self[self._key_use_index_url] = value

    @property
    def package_names(self):
        if self._key_package_names not in self:
            self[self._key_package_names] = list()
        return self[self._key_package_names]

    @package_names.setter
    def package_names(self, value: list):
        assert isinstance(value, list)
        self[self._key_package_names] = value

    @property
    def window_size(self):
        if self._key_window_size not in self:
            self[self._key_window_size] = 960, 600
        return self[self._key_window_size]

    @window_size.setter
    def window_size(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_window_size] = value

    @property
    def install_winsize(self):
        if self._key_install_winsize not in self:
            self[self._key_install_winsize] = 400, 420
        return self[self._key_install_winsize]

    @install_winsize.setter
    def install_winsize(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_install_winsize] = value

    @property
    def input_winsize(self):
        if self._key_input_winsize not in self:
            self[self._key_input_winsize] = 580, 0
        return self[self._key_input_winsize]

    @input_winsize.setter
    def input_winsize(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_input_winsize] = value

    @property
    def output_side(self):
        if self._key_output_side not in self:
            self[self._key_output_side] = Linkage.Left
        return self[self._key_output_side]

    @output_side.setter
    def output_side(self, value):
        assert isinstance(value, Linkage)
        self[self._key_output_side] = value

    @property
    def output_winsize(self):
        if self._key_output_winsize not in self:
            self[self._key_output_winsize] = 350, 500
        return self[self._key_output_winsize]

    @output_winsize.setter
    def output_winsize(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_output_winsize] = value

    @property
    def query_name(self):
        return self.setdefault(self._key_query_name, "")

    @query_name.setter
    def query_name(self, value):
        assert isinstance(value, str)
        self[self._key_query_name] = value

    @property
    def query_mode(self):
        return self.setdefault(self._key_query_mode, QMode.Pkg2Imp)

    @query_mode.setter
    def query_mode(self, value):
        assert isinstance(value, QMode)
        self[self._key_query_mode] = value

    @property
    def query_case(self):
        return self.setdefault(self._key_query_case, False)

    @query_case.setter
    def query_case(self, value):
        assert isinstance(value, bool)
        self[self._key_query_case] = value

    @property
    def query_winsize(self):
        return self.setdefault(self._key_query_winsize, (330, 330))

    @query_winsize.setter
    def query_winsize(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_query_winsize] = value



# coding: utf-8

from .addenv_dialog import Ui_input_dialog
from .environ_chosen import Ui_environ_chosen
from .generic_output import Ui_generic_output
from .imports_check import Ui_imports_check
from .index_manager import Ui_index_manager
from .main_entrance import Ui_main_entrance
from .package_download import Ui_package_download
from .package_install import Ui_package_install
from .package_manager import Ui_package_manager
from .pyinstaller_tool import Ui_pyinstaller_tool
from .query_panel import Ui_query_panel
from .show_download import Ui_show_download

__all__ = [
    "Ui_environ_chosen",
    "Ui_generic_output",
    "Ui_imports_check",
    "Ui_index_manager",
    "Ui_input_dialog",
    "Ui_main_entrance",
    "Ui_package_download",
    "Ui_package_install",
    "Ui_package_manager",
    "Ui_pyinstaller_tool",
    "Ui_query_panel",
    "Ui_show_download",
]



# coding: utf-8

__doc__ = "项目目录下虚拟环境的工具包"

from os import listdir, path
from random import randint

from fastpip import *

_CMD_PREFIX = ("-m", "venv")
_VENVPREFIX = "venv_"
_VENV_CFGFILE = "pyvenv.cfg"


class VirtualEnv(PyEnv):
    def __init__(self, project_root=""):
        super().__init__("")
        self.__venv_exist = False
        self.__project = project_root

    @property
    def project(self):
        return self.__project if path.isdir(self.__project) else ""

    @project.setter
    def project(self, value):
        self.__project = value

    def __find_venv_file(self, dir_path):
        try:
            names = listdir(dir_path)
        except:
            return False
        for p in names:
            fullpath = path.join(dir_path, p)
            if path.exists(path.join(fullpath, _VENV_CFGFILE)):
                self.path = fullpath
                self.__venv_exist = True
                return self.__venv_exist
        return False

    def find_project_venv(self):
        if not self.project:
            return False
        if self.__find_venv_file(self.project):
            return True
        parent_dir = path.dirname(self.project)
        if path.samefile(parent_dir, self.project):
            return False
        if self.__find_venv_file(parent_dir):
            return True
        parent_dir = path.dirname(parent_dir)
        if path.samefile(parent_dir, self.project):
            return False
        return self.__find_venv_file(parent_dir)

    @property
    def venv_exists(self):
        return self.__venv_exist

    def create_project_venv(self, interpreter):
        if not self.project:
            return False
        while True:
            dir_name = "%s%d" % (_VENVPREFIX, randint(1000, 9999))
            venv_fullpath = path.join(self.__project, dir_name)
            if not path.exists(venv_fullpath):
                break
        cmds = Command(interpreter, *_CMD_PREFIX, venv_fullpath)
        strings, return_code = execute_commands(cmds, False, None)
        if return_code:
            return False
        self.__venv_exist = True
        self.path = venv_fullpath
        return self.__venv_exist



# coding: utf-8

from os import path
from typing import Union

from com import *
from fastpip import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from settings import *
from ui import *
from utils import *
from utils.widgets import TextEdit

from .generic_output import GenericOutputWindow
from .messagebox import MessageBox
from .query_file_path import QueryFilePath


class PackageManagerWindow(Ui_package_manager, QMainWindow):
    def __init__(self, parent):
        super().__init__(parent)
        self.setupUi(self)
        self.config = PackageManagerConfig()
        self.__output = GenericOutputWindow(self)
        self.__handle = PyEnv.register(self.__output.add_line)
        self.__setup_other_widgets()
        self.signal_slot_connection()
        self.env_list = [PyEnv(p) for p in self.config.pypaths]
        self.__cur_pkgs_info = dict()
        self.__reverseds = [True, True, True, True]
        self.selected_index = -1
        self.__is_busy = False
        self.thread_repo = ThreadRepo(500)

    def __setup_other_widgets(self):
        self.tw_installed_info.setColumnWidth(0, 220)
        horiz_head = self.tw_installed_info.horizontalHeader()
        horiz_head.setSectionResizeMode(0, QHeaderView.Interactive)
        horiz_head.setSectionResizeMode(1, QHeaderView.ResizeToContents)
        horiz_head.setSectionResizeMode(2, QHeaderView.ResizeToContents)
        horiz_head.setSectionResizeMode(3, QHeaderView.Stretch)
        self.loading_mov = QMovie(":/loading.gif")
        self.loading_mov.setScaledSize(QSize(16, 16))

    def __move_output(self):
        if self.__output.isHidden() or self.__output.linkage == Linkage.NoLink:
            return
        fgeo = self.frameGeometry()
        geo = self.geometry()
        if self.__output.linkage == Linkage.Top:
            point = fgeo.bottomLeft()
            width = geo.width()
            height = None
        elif self.__output.linkage == Linkage.Left:
            point = fgeo.topRight()
            width = None
            height = geo.height()
        elif self.__output.linkage == Linkage.Right:
            point = fgeo.topLeft()
            width = None
            height = geo.height()
        else:
            return
        self.__output.set_geometry(point, width, height)

    def moveEvent(self, event: QMoveEvent):
        self.__move_output()

    def resizeEvent(self, event: QResizeEvent):
        self.__move_output()

    def display(self):
        self.resize(*self.config.window_size)
        if self.isMaximized():
            self.showMaximized()
        else:
            self.showNormal()
        self.list_widget_pyenvs_update()
        self.lw_env_list.setCurrentRow(self.selected_index)

    def show_hide_output(self):
        if self.__output.isHidden():
            if self.__output.not_shown_yet():
                self.__output.resize(*self.config.output_winsize)
                self.__output.showNormal()
                self.__output.linkage = Linkage(self.config.output_side)
                if self.__output.linkage != Linkage.NoLink:
                    self.__move_output()
            else:
                self.__output.showNormal()
                self.__move_output()
            self.__output.clear_content()
        else:
            self.__output.hide()

    @staticmethod
    def __stop_before_close():
        return not MessageBox(
            "警告",
            "当前有任务正在运行！\n是否尝试停止所有正在运行的任务并关闭窗口？",
            QMessageBox.Question,
            (("accept", "尝试停止并关闭"), ("reject", "取消")),
        ).exec_()

    def __save_window_size(self):
        if not (self.isMaximized() or self.isMinimized()):
            self.config.window_size = self.width(), self.height()
        if not (self.__output.isMaximized() or self.__output.isMinimized()):
            self.config.output_winsize = self.__output.width(), self.__output.height()

    def closeEvent(self, event: QCloseEvent):
        if not self.thread_repo.is_empty():
            if self.__stop_before_close():
                self.thread_repo.stop_all()
                self.__output.close()
                self.environ_changed_clear_pkgs()
                event.accept()
            else:
                event.ignore()
        else:
            self.__output.close()
        self.__save_window_size()
        if not self.__output.not_shown_yet():
            self.config.output_side = self.__output.linkage
        self.config.save_config()

    def show_loading(self, text):
        self.lb_loading_tip.clear()
        self.lb_loading_tip.setText(text)
        self.lb_loading_gif.clear()
        self.lb_loading_gif.setMovie(self.loading_mov)
        self.loading_mov.start()

    def hide_loading(self):
        self.loading_mov.stop()
        self.lb_loading_gif.clear()
        self.lb_loading_tip.clear()

    def signal_slot_connection(self):
        self.btn_autosearch.clicked.connect(self.auto_search_environ)
        self.btn_delselected.clicked.connect(self.del_selected_environ)
        self.btn_addmanully.clicked.connect(self.add_environ_manully)
        self.cb_check_uncheck_all.clicked.connect(self.selectall_unselectall)
        self.lw_env_list.clicked.connect(lambda: self.get_pkgs_info(0))
        self.lw_env_list.currentRowChanged.connect(self.environ_changed_clear_pkgs)
        self.btn_check_for_updates.clicked.connect(self.check_cur_pkgs_for_updates)
        self.btn_install_package.clicked.connect(self.set_win_install_package_envinfo)
        self.btn_uninstall_package.clicked.connect(self.uninstall_packages)
        self.btn_upgrade_package.clicked.connect(self.upgrade_packages)
        self.btn_upgrade_all.clicked.connect(self.upgrade_all_packages)
        self.tw_installed_info.horizontalHeader().sectionClicked[int].connect(
            self.__sort_by_column
        )
        self.tw_installed_info.clicked.connect(self.__show_label_selected_num)
        self.le_search_pkgs_kwd.textChanged.connect(self.search_pkg_name_by_kwd)
        self.lw_env_list.customContextMenuRequested.connect(
            self.environlist_contextmenu
        )
        self.tw_installed_info.customContextMenuRequested.connect(
            self.packagesinfo_contextmenu
        )
        self.uiPushButton_show_output.clicked.connect(self.show_hide_output)

    def open_selected_envfolder(self):
        index = self.lw_env_list.currentRow()
        environ_path = self.env_list[index].env_path
        if not path.isdir(environ_path):
            return MessageBox("提示", "所选环境目录不存在！").exec_()
        open_explorer(environ_path, "root")

    def query_names(self):
        if self.selected_index == -1:
            return MessageBox("提示", "还没有选中任何环境！").exec_()
        pkg_names = self.package_names_selected()
        if not pkg_names:
            pkg_name = None
            mode = QMode.NotSPCF
        else:
            pkg_name = pkg_names[0]
            mode = QMode.Pkg2Imp
        environ = self.env_list[self.selected_index]
        query_window = NameQueryPanel(self)
        query_window.initialize(environ, pkg_name, mode=mode)
        query_window.display()
        if pkg_name is not None:
            query_window.start_query_name()

    def environlist_contextmenu(self):
        contextmenu = QMenu(self)

        action = QAction(QIcon(":/openfd.png"), "打开", self)
        action.triggered.connect(self.open_selected_envfolder)
        contextmenu.addAction(action)

        contextmenu.addSeparator()

        action = QAction(QIcon(":/add.png"), "添加", self)
        action.triggered.connect(self.add_environ_manully)
        contextmenu.addAction(action)

        action = QAction(QIcon(":/search.png"), "搜索", self)
        action.triggered.connect(self.auto_search_environ)
        contextmenu.addAction(action)

        contextmenu.addSeparator()

        action = QAction(QIcon(":/delete.png"), "移除", self)
        action.triggered.connect(self.del_selected_environ)
        contextmenu.addAction(action)

        contextmenu.setStyleSheet("QMenu {padding: 10px; border: 1px solid black}")
        contextmenu.exec_(QCursor.pos())

    def packagesinfo_contextmenu(self):
        contextmenu = QMenu(self)
        action_list = list()

        action = QAction(QIcon(":/upgrade.png"), "升级", self)
        action.triggered.connect(self.upgrade_packages)
        action_list.append(action)
        contextmenu.addAction(action)

        action = QAction(QIcon(":/uninstall.png"), "卸载", self)
        action.triggered.connect(self.uninstall_packages)
        action_list.append(action)
        contextmenu.addAction(action)

        action = QAction(QIcon(":/query.png"), "查询", self)
        action.triggered.connect(self.query_names)
        action_list.append(action)
        contextmenu.addAction(action)

        action = QAction(QIcon(":/refresh.png"), "刷新", self)
        action.triggered.connect(lambda: self.get_pkgs_info(0))
        action_list.append(action)
        contextmenu.addAction(action)

        for action in action_list:
            if self.__is_busy:
                action.setEnabled(False)
            else:
                action.setEnabled(True)

        contextmenu.setStyleSheet("QMenu {padding: 10px; border: 1px solid black}")
        contextmenu.exec_(QCursor.pos())

    def set_win_install_package_envinfo(self):
        if not self.env_list:
            return MessageBox("提示", "没有可选的 Python 环境。").exec_()
        if self.selected_index <= -1:
            return MessageBox("提示", "没有选择任何 Python 环境。").exec_()
        if self.selected_index >= len(self.env_list):
            return MessageBox("错误", "异常：当前选择下标超出范围。").exec_()
        PackageInstallWindow(self, self.install_packages).set_target_environ(
            self.env_list[self.selected_index]
        )

    @staticmethod
    def judging_inclusion_relationship(string_long, keyword):
        string_long = string_long.lower()
        keyword = keyword.strip().lower()
        if len(keyword) < 2:
            return string_long.startswith(keyword)
        else:
            initial = keyword[0]
            letters_following = keyword[1:].strip()
            if not string_long.startswith(initial):
                return keyword in string_long
            return letters_following in string_long[1:]

    def search_pkg_name_by_kwd(self):
        keyword = self.le_search_pkgs_kwd.text()
        if not keyword:
            for i in range(self.tw_installed_info.rowCount()):
                self.tw_installed_info.showRow(i)
        else:
            for i in range(self.tw_installed_info.rowCount()):
                if self.judging_inclusion_relationship(
                    self.tw_installed_info.item(i, 0).text(), keyword
                ):
                    self.tw_installed_info.showRow(i)
                else:
                    self.tw_installed_info.hideRow(i)
        self.__show_label_selected_num(False)
        # 搜索功能比较简单，关键字是单独一个字母时，检索以该字母开头的模块
        # 如果不是单字母，则判断包名是否以关键词首字母开头、包名是否包含后续单词

    def __show_label_selected_num(self, clear=True):
        selected = len(self.indexs_of_selected_rows())
        if selected != len(self.__cur_pkgs_info):
            self.cb_check_uncheck_all.setChecked(False)
        else:
            self.cb_check_uncheck_all.setChecked(True)
        if clear and selected == 0:
            self.lb_num_selected_items.clear()
            return
        self.lb_num_selected_items.setText(f"选中数量：{selected}")

    def list_widget_pyenvs_update(self):
        row_size = QSize(0, 28)
        cur_py_env_index = self.lw_env_list.currentRow()
        self.lw_env_list.clear()
        for env in self.env_list:
            item = QListWidgetItem(str(env))
            item.setSizeHint(row_size)
            self.lw_env_list.addItem(item)
        if cur_py_env_index != -1:
            self.lw_env_list.setCurrentRow(cur_py_env_index)

    def table_widget_pkgs_info_update(self):
        self.lb_num_selected_items.clear()
        self.tw_installed_info.clearContents()
        self.tw_installed_info.setRowCount(len(self.__cur_pkgs_info))
        color_green = QColor(0, 170, 0)
        color_red = QColor(255, 0, 0)
        color_gray = QColor(243, 243, 243)
        for rowind, pkg_name in enumerate(self.__cur_pkgs_info):
            self.tw_installed_info.setVerticalHeaderItem(
                rowind, QTableWidgetItem(f" {rowind + 1} ")
            )
            item0 = QTableWidgetItem(f"{pkg_name}")
            self.tw_installed_info.setItem(rowind, 0, item0)
            even_num_row = rowind % 2
            if not even_num_row:
                item0.setBackground(color_gray)
            for colind, item_text in enumerate(
                self.__cur_pkgs_info.get(pkg_name, ["", "", ""])
            ):
                item = QTableWidgetItem(f" {item_text} ")
                if colind == 2:
                    if item_text in ("升级成功", "安装成功", "卸载成功"):
                        item.setForeground(color_green)
                    elif item_text in ("升级失败", "安装失败", "卸载失败"):
                        item.setForeground(color_red)
                    item.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
                if not even_num_row:
                    item.setBackground(color_gray)
                self.tw_installed_info.setItem(rowind, colind + 1, item)
        self.__is_busy = False

    def __sort_by_column(self, colind):
        if colind == 0:
            self.__cur_pkgs_info = dict(
                sorted(
                    self.__cur_pkgs_info.items(),
                    key=lambda x: x[0].lower(),
                    reverse=self.__reverseds[colind],
                )
            )
        else:
            self.__cur_pkgs_info = dict(
                sorted(
                    self.__cur_pkgs_info.items(),
                    key=lambda x: x[1][colind - 1],
                    reverse=self.__reverseds[colind],
                )
            )
        self.table_widget_pkgs_info_update()
        self.__reverseds[colind] = not self.__reverseds[colind]

    def environ_changed_clear_pkgs(self):
        if self.tw_installed_info.rowCount():
            self.lb_num_selected_items.clear()
            self.tw_installed_info.clearContents()
            self.tw_installed_info.setRowCount(0)
        self.__cur_pkgs_info.clear()
        self.selected_index = self.lw_env_list.currentRow()

    def get_pkgs_info(self, no_connect):
        if self.__is_busy:
            return
        self.__is_busy = True
        self.selected_index = self.lw_env_list.currentRow()
        if self.selected_index == -1:
            return None

        def do_get_pkgs_info():
            pkgs_info = self.env_list[self.selected_index].pkgs_info()
            self.__cur_pkgs_info.clear()
            for pkg_info in pkgs_info:
                self.__cur_pkgs_info[pkg_info[0]] = [pkg_info[1], "", ""]

        thread_get_pkgs_info = QThreadModel(do_get_pkgs_info)
        if not no_connect:
            thread_get_pkgs_info.at_start(
                self.lock_widgets,
                lambda: self.show_loading("正在加载包信息..."),
            )
            thread_get_pkgs_info.at_finish(
                self.table_widget_pkgs_info_update,
                self.hide_loading,
                self.release_widgets,
            )
        thread_get_pkgs_info.start()
        self.thread_repo.put(thread_get_pkgs_info, 1)
        return thread_get_pkgs_info

    def indexs_of_selected_rows(self):
        selected_row_indexs = [
            item.row() for item in self.tw_installed_info.selectedItems()[::4]
        ]
        selected_row_indexs.sort()
        return selected_row_indexs

    def package_names_selected(self):
        pkg_keys = tuple(self.__cur_pkgs_info.keys())
        return [pkg_keys[i] for i in self.indexs_of_selected_rows()]

    def selectall_unselectall(self):
        if self.cb_check_uncheck_all.isChecked():
            self.tw_installed_info.selectAll()
        else:
            self.tw_installed_info.clearSelection()
        self.__show_label_selected_num()

    def auto_search_environ(self):
        def search_environ():
            for _path in all_py_paths():
                if _path.lower() in path_list_lower:
                    continue
                try:
                    env = PyEnv(_path)
                except Exception:
                    continue
                self.env_list.append(env)
                self.config.pypaths.append(env.env_path)

        path_list_lower = [p.lower() for p in self.config.pypaths]
        thread_search_envs = QThreadModel(search_environ)
        thread_search_envs.at_start(
            self.lock_widgets,
            lambda: self.show_loading("正在搜索 Python 安装目录..."),
        )
        thread_search_envs.at_finish(
            self.environ_changed_clear_pkgs,
            self.list_widget_pyenvs_update,
            self.hide_loading,
            self.release_widgets,
        )
        thread_search_envs.start()
        self.thread_repo.put(thread_search_envs, 0)

    def del_selected_environ(self):
        cur_index = self.lw_env_list.currentRow()
        if cur_index == -1:
            return
        del self.env_list[cur_index]
        del self.config.pypaths[cur_index]
        self.lw_env_list.removeItemWidget(self.lw_env_list.takeItem(cur_index))
        self.environ_changed_clear_pkgs()

    def add_path_callback(self, _path):
        if not _path:
            return
        if not check_py_path(_path):
            return MessageBox(
                "警告",
                "无效的 Python 目录路径！",
                QMessageBox.Warning,
            ).exec_()
        if _path.lower() in [p.lower() for p in self.config.pypaths]:
            return MessageBox(
                "警告",
                "要添加的 Python 目录已存在。",
                QMessageBox.Warning,
            ).exec_()
        try:
            env = PyEnv(_path)
            self.env_list.append(env)
            self.config.pypaths.append(env.env_path)
        except Exception:
            return MessageBox(
                "警告",
                "目录添加失败，路径参数类型异常，请向开发者反馈~",
                QMessageBox.Warning,
            ).exec_()
        self.list_widget_pyenvs_update()

    def add_environ_manully(self):
        AddEnvironDialog(self, self.add_path_callback, "添加环境")

    def check_cur_pkgs_for_updates(self):
        if self.tw_installed_info.rowCount() == 0:
            return
        cur_row = self.lw_env_list.currentRow()
        if cur_row == -1:
            return
        thread_get_info = self.get_pkgs_info(no_connect=1)

        def do_get_outdated():
            thread_get_info.wait()
            outdateds = self.env_list[cur_row].outdated()
            for outdated_info in outdateds:
                self.__cur_pkgs_info.setdefault(outdated_info[0], ["", "", ""])[
                    1
                ] = outdated_info[2]

        thread_get_outdated = QThreadModel(do_get_outdated)
        thread_get_outdated.at_start(
            self.lock_widgets,
            lambda: self.show_loading("正在检查更新..."),
        )
        thread_get_outdated.at_finish(
            self.table_widget_pkgs_info_update,
            self.hide_loading,
            self.release_widgets,
        )
        thread_get_outdated.start()
        self.thread_repo.put(thread_get_outdated, 1)

    def lock_widgets(self):
        for widget in (
            self.btn_autosearch,
            self.btn_addmanully,
            self.btn_delselected,
            self.lw_env_list,
            self.cb_check_uncheck_all,
            self.btn_check_for_updates,
            self.btn_install_package,
            self.btn_uninstall_package,
            self.btn_upgrade_package,
            self.btn_upgrade_all,
            self.lb_num_selected_items,
        ):
            widget.setEnabled(False)

    def release_widgets(self):
        for widget in (
            self.btn_autosearch,
            self.btn_addmanully,
            self.btn_delselected,
            self.lw_env_list,
            self.cb_check_uncheck_all,
            self.btn_check_for_updates,
            self.btn_install_package,
            self.btn_uninstall_package,
            self.btn_upgrade_package,
            self.btn_upgrade_all,
            self.lb_num_selected_items,
        ):
            widget.setEnabled(True)

    def install_packages(self, cur_env, tobe_installed):
        if not (cur_env and tobe_installed):
            return
        include_pre = self.config.include_pre
        user = self.config.install_for_user
        use_index_url = self.config.use_index_url
        index_url = self.config.index_url if use_index_url else ""

        def do_install():
            installed = [
                [name, result]
                for name, result in loop_install(
                    cur_env,
                    tobe_installed,
                    pre=include_pre,
                    user=user,
                    index_url=index_url,
                )
            ]
            separated = parse_package_names(i[0] for i in installed)
            for i, value in enumerate(installed):
                value[0] = separated[i]
            for name, result in installed:
                item = self.__cur_pkgs_info.setdefault(name, ["", "", ""])
                item[0] = "N/A"
                item[2] = "安装成功" if result else "安装失败"

        thread_install_pkgs = QThreadModel(do_install)
        thread_install_pkgs.at_start(
            self.lock_widgets,
            lambda: self.show_loading("正在安装..."),
        )
        thread_install_pkgs.at_finish(
            self.table_widget_pkgs_info_update,
            self.hide_loading,
            self.release_widgets,
        )
        thread_install_pkgs.start()
        self.thread_repo.put(thread_install_pkgs, 0)

    def uninstall_packages(self):
        pkgs_info_keys = tuple(self.__cur_pkgs_info.keys())
        pkg_indexs = self.indexs_of_selected_rows()
        pkg_names = [pkgs_info_keys[index] for index in pkg_indexs]
        if not pkg_names:
            return MessageBox("提示", "没有选中任何项！").exec_()
        cur_env = self.env_list[self.lw_env_list.currentRow()]
        names_text = (
            "\n".join(pkg_names)
            if len(pkg_names) <= 10
            else "\n".join(("\n".join(pkg_names[:10]), "......"))
        )
        uninstall_msg_box = QMessageBox(
            QMessageBox.Question, "卸载", f"确认卸载？\n{names_text}"
        )
        uninstall_msg_box.addButton("确定", QMessageBox.AcceptRole)
        reject = uninstall_msg_box.addButton("取消", QMessageBox.RejectRole)
        uninstall_msg_box.setDefaultButton(reject)
        if uninstall_msg_box.exec_() != 0:
            return

        def do_uninstall():
            for pkg_name, code in loop_uninstall(cur_env, pkg_names):
                item = self.__cur_pkgs_info.setdefault(pkg_name, ["", "", ""])
                if code:
                    item[0] = "N/A"
                item[2] = "卸载成功" if code else "卸载失败"

        thread_uninstall_pkgs = QThreadModel(do_uninstall)
        thread_uninstall_pkgs.at_start(
            self.lock_widgets,
            lambda: self.show_loading("正在卸载..."),
        )
        thread_uninstall_pkgs.at_finish(
            self.table_widget_pkgs_info_update,
            self.hide_loading,
            self.release_widgets,
        )
        thread_uninstall_pkgs.start()
        self.thread_repo.put(thread_uninstall_pkgs, 0)

    def upgrade_packages(self):
        pkgs_info_keys = tuple(self.__cur_pkgs_info.keys())
        pkg_indexs = self.indexs_of_selected_rows()
        names = [pkgs_info_keys[index] for index in pkg_indexs]
        if not names:
            return MessageBox("提示", "没有选中任何项！").exec_()
        cur_env = self.env_list[self.lw_env_list.currentRow()]
        names_text = (
            "\n".join(names)
            if len(names) <= 10
            else "\n".join(("\n".join(names[:10]), "......"))
        )
        if (
            MessageBox(
                "升级",
                f"确认升级？\n{names_text}",
                QMessageBox.Question,
                (("accept", "确定"), ("reject", "取消")),
            ).exec_()
            != 0
        ):
            return

        def do_upgrade():
            for pkg, res in loop_install(cur_env, names, upgrade=True):
                item = self.__cur_pkgs_info.setdefault(pkg, ["", "", ""])
                if res:
                    item[2] = "升级成功"
                    if item[1]:
                        item[0] = item[1]
                    else:
                        item[0] = "N/A"
                else:
                    item[2] = "升级失败"

        thread_upgrade_pkgs = QThreadModel(do_upgrade)
        thread_upgrade_pkgs.at_start(
            self.lock_widgets,
            lambda: self.show_loading("正在升级..."),
        )
        thread_upgrade_pkgs.at_finish(
            self.table_widget_pkgs_info_update,
            self.hide_loading,
            self.release_widgets,
        )
        thread_upgrade_pkgs.start()
        self.thread_repo.put(thread_upgrade_pkgs, 0)

    def upgrade_all_packages(self):
        upgradeable = [i[0] for i in self.__cur_pkgs_info.items() if i[1][1]]
        if not upgradeable:
            MessageBox(
                "提示",
                "请检查更新确认是否有可更新的包。",
                QMessageBox.Information,
            ).exec_()
            return
        cur_env = self.env_list[self.lw_env_list.currentRow()]
        names_text = (
            "\n".join(upgradeable)
            if len(upgradeable) <= 10
            else "\n".join(("\n".join(upgradeable[:10]), "......"))
        )
        if (
            MessageBox(
                "全部升级",
                f"确认升级？\n{names_text}",
                QMessageBox.Question,
                (("accept", "确定"), ("reject", "取消")),
            ).exec_()
            != 0
        ):
            return

        def do_upgrade():
            for pkg_name, code in loop_install(cur_env, upgradeable, upgrade=True):
                item = self.__cur_pkgs_info.setdefault(pkg_name, ["", "", ""])
                if code and item[1]:
                    item[0] = item[1]
                item[2] = "升级成功" if code else "升级失败"

        thread_upgrade_pkgs = QThreadModel(do_upgrade)
        thread_upgrade_pkgs.at_start(
            self.lock_widgets,
            lambda: self.show_loading("正在升级..."),
        )
        thread_upgrade_pkgs.at_finish(
            self.table_widget_pkgs_info_update,
            self.hide_loading,
            self.release_widgets,
        )
        thread_upgrade_pkgs.start()
        self.thread_repo.put(thread_upgrade_pkgs, 0)


class PackageInstallWindow(Ui_package_install, QMainWindow, QueryFilePath):
    def __init__(self, parent: PackageManagerWindow, back):
        super().__init__(parent)
        self.setupUi(self)
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)
        self.__setup_other_widgets()
        self.environment = None
        self.package_names = list()
        self.signal_slot_connection()
        self.__parent = parent
        self.__callback = back
        self.last_path = self.__parent.config.last_path

    def __setup_other_widgets(self):
        self.pte_package_names = TextEdit(ext_filter={".whl"})
        self.uiHorizontalLayout_package_name.replaceWidget(
            self.pte_package_names_old, self.pte_package_names
        )
        self.pte_package_names.show()
        self.pte_package_names_old.deleteLater()

    def display(self):
        self.resize(*self.__parent.config.install_winsize)
        self.showNormal()

    def save_package_names(self):
        data = self.pte_package_names.toPlainText()
        if not data:
            MessageBox("提示", "要保存的内容为空！").exec_()
            return
        last_path = self.save_as_text_file(data, self.last_path)
        if last_path:
            self.last_path = last_path
            self.__parent.config.last_path = last_path

    def load_package_names(self):
        text, fpath = self.load_from_text(self.last_path)
        if text and fpath:
            self.__parent.config.last_path = fpath
            self.last_path = fpath
            self.pte_package_names.setPlainText(text)

    def config_dict_to_widgets(self):
        self.pte_package_names.setPlainText(
            "\n".join(self.__parent.config.package_names)
        )
        self.cb_including_pre.setChecked(self.__parent.config.include_pre)
        self.cb_install_for_user.setChecked(self.__parent.config.install_for_user)
        self.le_use_index_url.setText(self.__parent.config.index_url)
        self.cb_use_index_url.setChecked(self.__parent.config.use_index_url)
        if self.cb_use_index_url.isChecked():
            self.le_use_index_url.setEnabled(True)
        else:
            self.le_use_index_url.setEnabled(False)

    def config_widgets_to_dict(self):
        self.package_names.extend(
            s for s in self.pte_package_names.toPlainText().splitlines() if s
        )
        self.__parent.config.package_names = self.package_names.copy()
        self.__parent.config.include_pre = self.cb_including_pre.isChecked()
        self.__parent.config.install_for_user = self.cb_install_for_user.isChecked()
        self.__parent.config.index_url = self.le_use_index_url.text()
        self.__parent.config.use_index_url = self.cb_use_index_url.isChecked()

    def call_installpkg_back(self):
        self.close()  # 触发 closeEvent 更新配置
        self.__callback(self.environment, self.package_names)

    def set_target_environ(self, env):
        self.environment = env
        self.config_dict_to_widgets()
        self.uiLabel_target_environment.setText(str(env))
        self.display()

    def __save_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__parent.config.install_winsize = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        self.__save_window_size()
        self.config_widgets_to_dict()

    def signal_slot_connection(self):
        self.pb_do_install.clicked.connect(self.call_installpkg_back)
        self.pb_save_as_text.clicked.connect(self.save_package_names)
        self.pb_load_from_text.clicked.connect(self.load_package_names)
        self.cb_use_index_url.clicked.connect(self.set_le_use_index_url)

    def set_le_use_index_url(self):
        self.le_use_index_url.setEnabled(self.cb_use_index_url.isChecked())


class AddEnvironDialog(Ui_input_dialog, QMainWindow):
    def __init__(self, parent: PackageManagerWindow, back, title=""):
        self.__parent = parent
        super().__init__(parent)
        self.setupUi(self)
        self.initialize()
        if title:
            self.setWindowTitle(title)
        self.__callback = back
        self.last_path = parent.config.last_path
        self.display()

    def display(self):
        self.resize(*self.__parent.config.input_winsize)
        self.showNormal()

    def initialize(self):
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)
        self.uiPushButton_confirm.clicked.connect(self.__text_back)
        self.uiPushButton_select_envdir.clicked.connect(self.__select_envdir)

    def keyPressEvent(self, event: QKeyEvent):
        key = event.key()
        if key == Qt.Key_Escape:
            self.close()
        elif key == Qt.Key_Enter or key == Qt.Key_Return:
            self.__text_back()

    def __text_back(self):
        self.close()
        self.__callback(self.uiLineEdit_input_content.text())

    def __save_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__parent.config.input_winsize = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        self.__save_window_size()

    def __select_envdir(self):
        _path = QFileDialog.getExistingDirectory(self, "选择文件夹", self.last_path)
        if not _path:
            return
        interpreter_directory = path.normpath(_path)
        self.__parent.config.last_path = interpreter_directory
        self.uiLineEdit_input_content.setText(interpreter_directory)


class NameQueryPanel(Ui_query_panel, QMainWindow):
    signal_result = pyqtSignal(str)

    def __init__(self, parent: PackageManagerWindow):
        self.__parent = parent
        super().__init__(parent)
        self.setupUi(self)
        self.__initialize_widgets()
        self.__environ: Union[PyEnv, None] = None

    def eventFilter(self, obj: QObject, event: QEvent) -> bool:
        if obj == self.uiLineEdit_input_name:
            if event.type() == QEvent.FocusIn:
                QTimer.singleShot(10, self.uiLineEdit_input_name.selectAll)
            elif event.type() == QEvent.FocusOut:
                self.uiLineEdit_input_name.deselect()
        return super().eventFilter(obj, event)

    def display(self):
        self.resize(*self.__parent.config.query_winsize)
        self.showNormal()

    def __initialize_widgets(self):
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)
        self.uiLineEdit_input_name.installEventFilter(self)
        self.signal_result.connect(self.uiPlainTextEdit_query_result.appendPlainText)
        self.uiPushButton_query.clicked.connect(self.start_query_name)

    def keyPressEvent(self, event: QKeyEvent):
        key = event.key()
        if key == Qt.Key_Escape:
            self.close()
        elif key == Qt.Key_Enter or key == Qt.Key_Return:
            self.start_query_name()

    def __save_query_configure(self):
        self.__parent.config.query_name = self.uiLineEdit_input_name.text()
        if self.uiRadioButton_pkg2import.isChecked():
            self.__parent.config.query_mode = QMode.Pkg2Imp
        elif self.uiRadioButton_import2pkg.isChecked():
            self.__parent.config.query_mode = QMode.Imp2Pkg
        self.__parent.config.query_case = self.uiCheckBox_case_sensitive.isChecked()
        if self.isMaximized() or self.isMinimized():
            return
        self.__parent.config.query_winsize = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        self.__save_query_configure()

    def __start_querying(self):
        self.uiPushButton_query.setEnabled(False)
        self.uiRadioButton_pkg2import.setEnabled(False)
        self.uiRadioButton_import2pkg.setEnabled(False)
        self.uiLabel_query_result.setText("正在查询...")

    def __end_of_query(self):
        self.uiPushButton_query.setEnabled(True)
        self.uiRadioButton_pkg2import.setEnabled(True)
        self.uiRadioButton_import2pkg.setEnabled(True)
        self.uiLabel_query_result.setText("查询结果：")

    def start_query_name(self):
        name = self.uiLineEdit_input_name.text()
        if self.__environ is None:
            return MessageBox("提示", "没有选择任何 Python 环境。").exec_()
        if not name:
            return
        self.uiPlainTextEdit_query_result.clear()
        case = self.uiCheckBox_case_sensitive.isChecked()
        self.__start_querying()

        def do_query():
            if self.uiRadioButton_pkg2import.isChecked():
                result = self.__environ.query_for_import(name, case=case)
                for res in result:
                    self.signal_result.emit(res)
            elif self.uiRadioButton_import2pkg.isChecked():
                result = self.__environ.query_for_install(name, case=case)
                self.signal_result.emit(result)

        thread_query = QThreadModel(do_query)
        thread_query.at_finish(self.__end_of_query)
        thread_query.start()
        self.__parent.thread_repo.put(thread_query, 1)

    def initialize(self, env: PyEnv, name: str = None, mode=QMode.NotSPCF):
        assert isinstance(env, PyEnv)
        assert isinstance(name, str) or name is None
        assert isinstance(mode, QMode)
        self.__environ = env
        self.uiLabel_query_environ.setText(str(env))
        self.uiCheckBox_case_sensitive.setChecked(self.__parent.config.query_case)
        if name is None:
            name = self.__parent.config.query_name
        self.uiLineEdit_input_name.setText(name)
        if mode == QMode.NotSPCF:
            if self.__parent.config == QMode.NotSPCF:
                mode = QMode.Pkg2Imp
            else:
                mode = self.__parent.config.query_mode
        if mode == QMode.Pkg2Imp:
            self.uiRadioButton_pkg2import.setChecked(True)
        elif mode == QMode.Imp2Pkg:
            self.uiRadioButton_import2pkg.setChecked(True)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'environ_chosen.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_environ_chosen(object):
    def setupUi(self, environ_chosen):
        environ_chosen.setObjectName("environ_chosen")
        environ_chosen.setWindowModality(QtCore.Qt.WindowModal)
        environ_chosen.resize(420, 200)
        environ_chosen.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.centralwidget = QtWidgets.QWidget(environ_chosen)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        self.centralwidget.setFont(font)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lw_environ_list = QtWidgets.QListWidget(self.centralwidget)
        self.lw_environ_list.setObjectName("lw_environ_list")
        self.verticalLayout.addWidget(self.lw_environ_list)
        self.label = QtWidgets.QLabel(self.centralwidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(91, 91, 91))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(91, 91, 91))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(120, 120, 120))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        self.label.setPalette(palette)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(9)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        environ_chosen.setCentralWidget(self.centralwidget)

        self.retranslateUi(environ_chosen)
        QtCore.QMetaObject.connectSlotsByName(environ_chosen)

    def retranslateUi(self, environ_chosen):
        _translate = QtCore.QCoreApplication.translate
        environ_chosen.setWindowTitle(_translate("environ_chosen", "选择 Python 环境"))
        self.label.setText(_translate("environ_chosen", "需要先在包管理器中添加环境，这里才会显示可供选择的项~"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'show_download.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_show_download(object):
    def setupUi(self, show_download):
        show_download.setObjectName("show_download")
        show_download.setWindowModality(QtCore.Qt.WindowModal)
        show_download.resize(260, 500)
        self.centralwidget = QtWidgets.QWidget(show_download)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        self.centralwidget.setFont(font)
        self.centralwidget.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tw_downloading = QtWidgets.QTableWidget(self.centralwidget)
        self.tw_downloading.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tw_downloading.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.tw_downloading.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tw_downloading.setCornerButtonEnabled(False)
        self.tw_downloading.setObjectName("tw_downloading")
        self.tw_downloading.setColumnCount(2)
        self.tw_downloading.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tw_downloading.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_downloading.setHorizontalHeaderItem(1, item)
        self.tw_downloading.horizontalHeader().setHighlightSections(False)
        self.tw_downloading.horizontalHeader().setStretchLastSection(True)
        self.tw_downloading.verticalHeader().setHighlightSections(False)
        self.verticalLayout.addWidget(self.tw_downloading)
        show_download.setCentralWidget(self.centralwidget)

        self.retranslateUi(show_download)
        QtCore.QMetaObject.connectSlotsByName(show_download)

    def retranslateUi(self, show_download):
        _translate = QtCore.QCoreApplication.translate
        show_download.setWindowTitle(_translate("show_download", "下载列表"))
        item = self.tw_downloading.horizontalHeaderItem(0)
        item.setText(_translate("show_download", "下载项"))
        item = self.tw_downloading.horizontalHeaderItem(1)
        item.setText(_translate("show_download", "下载状态"))



# coding: utf-8

import os

from fastpip import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from settings import *
from ui import *
from utils import *

from .messagebox import MessageBox
from .query_file_path import QueryFilePath


class PackageDownloadWindow(Ui_package_download, QMainWindow, QueryFilePath):
    set_download_table = pyqtSignal(list)
    download_completed = pyqtSignal(str)
    download_status = pyqtSignal(int, str)

    def __init__(self, parent):
        super().__init__(parent)
        self.setupUi(self)
        self.env_paths = None
        self.environments = None
        self.config = PackageDownloadConfig()
        self.__showdl_win = ShowDownloadWindow(self)
        self.signal_slot_connection()
        self.last_path = None
        self.thread_repo = ThreadRepo(500)

    def signal_slot_connection(self):
        self.cb_use_index_url.clicked.connect(self.change_le_index_url)
        self.pb_load_from_text.clicked.connect(self.names_from_file)
        self.pb_save_as_text.clicked.connect(self.save_names_to_file)
        self.pb_save_to.clicked.connect(self.select_saved_dir)
        self.pb_clear_package_names.clicked.connect(self.pte_package_names.clear)
        self.pb_start_download.clicked.connect(self.start_download_package)
        self.download_completed.connect(self.check_download)
        self.pb_show_dl_list.clicked.connect(self.__showdl_win.display)
        self.download_status.connect(self.__showdl_win.status_changed)
        self.set_download_table.connect(self.__showdl_win.setup_table)

    def change_le_index_url(self):
        self.le_index_url.setEnabled(self.cb_use_index_url.isChecked())

    def names_from_file(self):
        text, _path = self.load_from_text(self.last_path)
        if _path:
            self.last_path = _path
        if text:
            self.pte_package_names.setPlainText(text)

    def save_names_to_file(self):
        data = self.pte_package_names.toPlainText()
        _path = self.save_as_text_file(data, self.last_path)
        if _path:
            self.last_path = _path

    def select_saved_dir(self):
        dir_path = self.get_dir_path(self.last_path)
        if dir_path:
            self.last_path = dir_path
            self.le_save_to.setText(dir_path)

    def __store_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.config.window_size = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        if not self.thread_repo.is_empty():
            MessageBox(
                "警告",
                "有下载任务正在运行，关闭窗口并不会结束任务。",
                QMessageBox.Warning,
            ).exec_()
        self.__store_window_size()
        self.config_widgets_to_dict()
        self.config.save_config()

    def display(self):
        self.resize(*self.config.window_size)
        if self.isMaximized():
            self.showMaximized()
        else:
            self.showNormal()
        if self.thread_repo.is_empty():
            self.apply_config()

    def update_envpaths_and_combobox(self):
        if self.env_paths is None:
            self.env_paths = list()
        else:
            self.env_paths.clear()
        self.env_paths.extend(self.config.cur_pypaths)
        self.environments = [PyEnv(p) for p in self.env_paths]
        index = self.config.derived_from
        text_list = [str(e) for e in self.environments]
        if index < 0 or index >= len(text_list):
            index = 0
        self.cmb_derived_from.clear()
        self.cmb_derived_from.addItems(text_list)
        self.cmb_derived_from.setCurrentIndex(index)

    def config_widgets_to_dict(self):
        self.config.package_names = [
            s for s in self.pte_package_names.toPlainText().split("\n") if s
        ]
        self.config.derived_from = self.cmb_derived_from.currentIndex()
        self.config.download_deps = self.cb_download_deps.isChecked()
        download_type = (
            "unlimited"
            if self.rb_unlimited.isChecked()
            else "no_binary"
            if self.rb_no_binary.isChecked()
            else "only_binary"
            if self.rb_only_binary.isChecked()
            else "prefer_binary"
        )
        self.config.download_type = download_type
        self.config.include_pre = self.cb_include_pre.isChecked()
        self.config.ignore_requires_python = (
            self.cb_ignore_requires_python.isChecked()
        )
        self.config.save_path = self.le_save_to.text()
        self.config.platform = [s for s in self.le_platform.text().split() if s]
        self.config.python_version = self.le_python_version.text()
        self.config.implementation = self.cmb_implementation.currentText()
        self.config.abis = [s for s in self.le_abis.text().split() if s]
        self.config.index_url = self.le_index_url.text()
        self.config.use_index_url = self.cb_use_index_url.isChecked()

    def apply_config(self):
        self.update_envpaths_and_combobox()
        self.pte_package_names.setPlainText("\n".join(self.config.package_names))
        self.cb_download_deps.setChecked(self.config.download_deps)
        download_type = self.config.download_type
        if download_type == "unlimited":
            self.rb_unlimited.setChecked(True)
        elif download_type == "no_binary":
            self.rb_no_binary.setChecked(True)
        elif download_type == "only_binary":
            self.rb_only_binary.setChecked(True)
        elif download_type == "prefer_binary":
            self.rb_prefer_binary.setChecked(True)
        else:
            self.rb_unlimited.setChecked(True)
        self.cb_include_pre.setChecked(self.config.include_pre)
        self.cb_ignore_requires_python.setChecked(self.config.ignore_requires_python)
        self.le_save_to.setText(self.config.save_path)
        self.le_platform.setText(" ".join(self.config.platform))
        self.le_python_version.setText(self.config.python_version)
        self.cmb_implementation.setCurrentText(self.config.implementation)
        self.le_abis.setText(" ".join(self.config.abis))
        use_index_url = self.config.use_index_url
        self.cb_use_index_url.setChecked(use_index_url)
        self.le_index_url.setEnabled(use_index_url)
        self.le_index_url.setText(self.config.index_url)

    @staticmethod
    def confirm_dest_path(dest):
        # 保存位置未填写时
        if not dest:
            return True
        if not os.path.exists(dest):
            if (
                MessageBox(
                    "提示",
                    "保存目录不存在，是否创建目录？",
                    QMessageBox.Warning,
                    (("accept", "是"), ("reject", "否")),
                ).exec_()
                == 0
            ):
                try:
                    os.makedirs(dest)
                    return True
                except Exception as e:
                    MessageBox("提示", f"保存目录创建失败：\n{e}。").exec_()
                    return False
            else:
                return False
        elif os.path.isfile(dest):
            MessageBox("提示", "该位置已存在同名的文件，请修改目录路径。").exec_()
            return False
        return True

    def start_download_package(self):
        if not self.environments:
            return MessageBox(
                "提示",
                "没有任何 Python 环境，请到'包管理器'中自动或手动添加 Python 环境路径。",
            ).exec_()
        self.config_widgets_to_dict()
        destination = self.config.save_path
        pkg_names = self.config.package_names
        if not self.confirm_dest_path(destination):
            return
        if not pkg_names:
            return MessageBox(
                "提示",
                "没有需要下载的安装包。",
            ).exec_()
        index = self.config.derived_from
        if index < 0 or index >= len(self.environments):
            index = 0
            self.cmb_derived_from.setCurrentIndex(0)
        env = self.environments[index]
        if not env.env_path:
            return MessageBox(
                "提示",
                "无效的 Python 环境，请检查环境是否已卸载。",
            ).exec_()
        config = self.make_configure(pkg_names)
        if not isinstance(config, dict):
            return

        def do_download():
            saved_path = ""
            self.set_download_table.emit(pkg_names)
            for ind, name in enumerate(pkg_names):
                self.download_status.emit(ind, "下载中...")
                try:
                    status = env.download(name, **config)
                    if status[0]:
                        self.download_status.emit(ind, "下载完成")
                    else:
                        self.download_status.emit(ind, "下载失败")
                except Exception:
                    status = False, ""
                    self.download_status.emit(ind, "下载失败")
                if status[1]:
                    saved_path = status[1]
            self.download_completed.emit(saved_path)

        thread_download = QThreadModel(do_download)
        thread_download.at_start(lambda: self.pb_start_download.setEnabled(False))
        thread_download.at_finish(
            lambda: self.pb_start_download.setEnabled(True),
        )
        thread_download.start()
        self.thread_repo.put(thread_download, 0)

    @staticmethod
    def check_download(dest):
        if not dest:
            return MessageBox(
                "提示",
                f"安装包下载失败!",
                QMessageBox.Critical,
            ).exec_()
        return MessageBox(
            "下载结束",
            f"安装包保存位置：\n{dest}",
        ).exec_()

    def make_configure(self, names):
        configure = dict()

        def unqualified():
            return (
                not configure.get("no_deps", False)
                or configure.get("no_binary", False)
                or configure.get("only_binary", False)
            )

        if not self.config.download_deps:
            configure.update(no_deps=True)
        download_type = self.config.download_type
        if download_type == "no_binary":
            configure.update(no_binary=parse_package_names(names))
        elif download_type == "only_binary":
            configure.update(only_binary=parse_package_names(names))
        elif download_type == "prefer_binary":
            configure.update(prefer_binary=True)
        if self.config.include_pre:
            configure.update(pre=True)
        if self.config.ignore_requires_python:
            configure.update(ignore_requires_python=True)
        saved_path = self.config.save_path
        if saved_path:
            configure.update(dest=saved_path)
        platform_name = self.config.platform
        if platform_name:
            if unqualified():
                return MessageBox(
                    "提示",
                    "设置'兼容平台'后，不能勾选'下载需要下载的包的依赖库'，\
不能选择'仅选择源代码包'或'仅选择二进制包'。",
                    QMessageBox.Warning,
                ).exec_()
            configure.update(platform=platform_name)
        python_version = self.config.python_version
        if python_version:
            if unqualified():
                return MessageBox(
                    "提示",
                    "设置'兼容 Python 版本'后，不能勾选'下载需要下载的包的依赖库'，\
不能选择'仅选择源代码包'或'仅选择二进制包'。",
                    QMessageBox.Warning,
                ).exec_()
            configure.update(python_version=python_version)
        impl_name = self.config.implementation
        if impl_name == "无特定实现":
            impl_name = "py"
        if impl_name:
            if unqualified():
                return MessageBox(
                    "提示",
                    "设置'兼容解释器实现'后，不能勾选'下载需要下载的包的依赖库'，\
不能选择'仅选择源代码包'或'仅选择二进制包'。",
                    QMessageBox.Warning,
                ).exec_()
            configure.update(implementation=impl_name)
        abis = self.config.abis
        if abis:
            if unqualified():
                return MessageBox(
                    "提示",
                    "设置'兼容ABI'后，不能勾选'下载需要下载的包的依赖库'，\
不能选择'仅选择源代码包'或'仅选择二进制包'。",
                    QMessageBox.Warning,
                ).exec_()
            if not (platform_name and python_version and impl_name):
                return MessageBox(
                    "提示",
                    "当指定ABI时，通常需同时指定'兼容平台'、'兼容 Python 版本'、\
'兼容解释器实现'三个下载条件。",
                ).exec_()
            configure.update(abis=abis)
        index_url = self.config.index_url
        if self.config.use_index_url and index_url:
            configure.update(index_url=index_url)
        return configure


class ShowDownloadWindow(Ui_show_download, QMainWindow):
    def __init__(self, parent: PackageDownloadWindow):
        self.__parent = parent
        super().__init__(parent)
        self.setupUi(self)
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)
        self._setup_other_widgets()

    def status_changed(self, index, status):
        if index >= self.tw_downloading.rowCount():
            return False
        color_red = QColor(255, 0, 0)
        color_green = QColor(0, 170, 0)
        item = self.tw_downloading.item(index, 1)
        if item is None:
            item = QTableWidgetItem("等待下载")
            self.tw_downloading.setItem(index, 1, item)
        item.setText(status)
        if status == "下载失败":
            item.setForeground(color_red)
        elif status == "下载完成":
            item.setForeground(color_green)
        return True

    def clear_table(self):
        self.tw_downloading.clearContents()
        self.tw_downloading.setRowCount(0)

    def setup_table(self, iterable):
        color_gray = QColor(243, 243, 243)
        self.clear_table()
        self.tw_downloading.setRowCount(len(iterable))
        for index, pkg_name in enumerate(iterable):
            item1 = QTableWidgetItem(pkg_name)
            item2 = QTableWidgetItem("等待下载")
            if not index % 2:
                item1.setBackground(color_gray)
                item2.setBackground(color_gray)
            self.tw_downloading.setItem(index, 0, item1)
            self.tw_downloading.setItem(index, 1, item2)
        return True

    def _setup_other_widgets(self):
        horiz_head = self.tw_downloading.horizontalHeader()
        horiz_head.setSectionResizeMode(0, QHeaderView.Stretch)
        horiz_head.setSectionResizeMode(1, QHeaderView.ResizeToContents)

    def __store_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__parent.config.dlstatus_winsize = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        self.__store_window_size()

    def display(self):
        self.resize(*self.__parent.config.dlstatus_winsize)
        self.showNormal()



# coding: utf-8

from copy import deepcopy
from typing import Sequence

from .abstract_config import AbstractConfig
from .package_manager import get_shared_pypaths


class PyiConfigure(dict):
    _key_script_path = "script_path"
    _key_project_root = "project_root"
    _key_bundle_spec_name = "bundle_spec_name"
    _key_module_paths = "module_paths"
    _key_other_datas = "other_datas"
    _key_icon_path = "icon_path"
    _key_onedir_bundle = "onedir_bundle"
    _key_provide_console = "provide_console"
    _key_no_confirm = "no_confirm"
    _key_donot_use_upx = "donot_use_upx"
    _key_clean_building = "clean_building"
    _key_add_verfile = "add_verfile"
    _key_working_dir = "working_dir"
    _key_distribution_dir = "distribution_dir"
    _key_spec_dir = "spec_dir"
    _key_upx_dir = "upx_dir"
    _key_upx_excludes = "upx_excludes"
    _key_environ_path = "environ_path"
    _key_log_level = "log_level"
    _key_version_info = "version_info"
    _key_debug_options = "debug_options"
    _key_runtime_tmpdir = "runtime_tmpdir"
    _key_prioritize_venv = "prioritize_venv"
    _key_encryption_key = "encryption_key"
    _key_uac_admin = "uac_admin"
    _key_open_dist_folder = "open_dist_folder"
    _key_delete_spec_file = "delete_spec_file"
    _key_delete_working_dir = "delete_working_dir"
    _key_hidden_imports = "hidden_imports"
    _key_exclude_modules = "exclude_modules"

    def __init__(self):
        super().__init__()

    @property
    def script_path(self):
        return self.setdefault(self._key_script_path, "")

    @script_path.setter
    def script_path(self, value):
        assert isinstance(value, str)
        self[self._key_script_path] = value

    @property
    def project_root(self):
        return self.setdefault(self._key_project_root, "")

    @project_root.setter
    def project_root(self, value):
        assert isinstance(value, str)
        self[self._key_project_root] = value

    @property
    def bundle_spec_name(self):
        return self.setdefault(self._key_bundle_spec_name, "")

    @bundle_spec_name.setter
    def bundle_spec_name(self, value):
        assert isinstance(value, str)
        self[self._key_bundle_spec_name] = value

    @property
    def module_paths(self):
        return self.setdefault(self._key_module_paths, list())

    @module_paths.setter
    def module_paths(self, value):
        assert isinstance(value, list)
        self[self._key_module_paths] = value

    @property
    def other_datas(self):
        return self.setdefault(self._key_other_datas, list())

    @other_datas.setter
    def other_datas(self, value):
        assert isinstance(value, list)
        self[self._key_other_datas] = value

    @property
    def icon_path(self):
        return self.setdefault(self._key_icon_path, "")

    @icon_path.setter
    def icon_path(self, value):
        assert isinstance(value, str)
        self[self._key_icon_path] = value

    @property
    def onedir_bundle(self):
        return self.setdefault(self._key_onedir_bundle, True)

    @onedir_bundle.setter
    def onedir_bundle(self, value):
        assert isinstance(value, bool)
        self[self._key_onedir_bundle] = value

    @property
    def provide_console(self):
        return self.setdefault(self._key_provide_console, True)

    @provide_console.setter
    def provide_console(self, value):
        assert isinstance(value, bool)
        self[self._key_provide_console] = value

    @property
    def no_confirm(self):
        return self.setdefault(self._key_no_confirm, False)

    @no_confirm.setter
    def no_confirm(self, value):
        assert isinstance(value, bool)
        self[self._key_no_confirm] = value

    @property
    def donot_use_upx(self):
        return self.setdefault(self._key_donot_use_upx, False)

    @donot_use_upx.setter
    def donot_use_upx(self, value):
        assert isinstance(value, bool)
        self[self._key_donot_use_upx] = value

    @property
    def clean_building(self):
        return self.setdefault(self._key_clean_building, False)

    @clean_building.setter
    def clean_building(self, value):
        assert isinstance(value, bool)
        self[self._key_clean_building] = value

    @property
    def add_verfile(self):
        return self.setdefault(self._key_add_verfile, False)

    @add_verfile.setter
    def add_verfile(self, value):
        assert isinstance(value, bool)
        self[self._key_add_verfile] = value

    @property
    def working_dir(self):
        return self.setdefault(self._key_working_dir, "")

    @working_dir.setter
    def working_dir(self, value):
        assert isinstance(value, str)
        self[self._key_working_dir] = value

    @property
    def distribution_dir(self):
        return self.setdefault(self._key_distribution_dir, "")

    @distribution_dir.setter
    def distribution_dir(self, value):
        assert isinstance(value, str)
        self[self._key_distribution_dir] = value

    @property
    def spec_dir(self):
        return self.setdefault(self._key_spec_dir, "")

    @spec_dir.setter
    def spec_dir(self, value):
        assert isinstance(value, str)
        self[self._key_spec_dir] = value

    @property
    def upx_dir(self):
        return self.setdefault(self._key_upx_dir, "")

    @upx_dir.setter
    def upx_dir(self, value):
        assert isinstance(value, str)
        self[self._key_upx_dir] = value

    @property
    def upx_excludes(self):
        return self.setdefault(self._key_upx_excludes, list())

    @upx_excludes.setter
    def upx_excludes(self, value):
        assert isinstance(value, list)
        self[self._key_upx_excludes] = value

    @property
    def environ_path(self):
        return self.setdefault(self._key_environ_path, "")

    @environ_path.setter
    def environ_path(self, value):
        assert isinstance(value, str)
        self[self._key_environ_path] = value

    @property
    def log_level(self):
        return self.setdefault(self._key_log_level, "INFO")

    @log_level.setter
    def log_level(self, value):
        assert value in ("TRACE", "DEBUG", "INFO", "WARN", "ERROR", "CRITICAL")
        self[self._key_log_level] = value

    @property
    def version_info(self):
        default = {
            "$filevers$": "(0, 0, 0, 0)",
            "$prodvers$": "(0, 0, 0, 0)",
            "$CompanyName$": "",
            "$FileDescription$": "",
            "$LegalCopyright$": "",
            "$OriginalFilename$": "",
            "$ProductName$": "",
            "$FileVersion$": "0.0.0.0",
            "$ProductVersion$": "0.0.0.0",
            "$LegalTrademarks$": "",
        }
        return self.setdefault(self._key_version_info, default)

    @version_info.setter
    def version_info(self, value):
        assert isinstance(value, dict)
        self[self._key_version_info] = value

    @property
    def debug_options(self):
        default = {"imports": False, "bootloader": False, "noarchive": False}
        return self.setdefault(self._key_debug_options, default)

    @debug_options.setter
    def debug_options(self, value):
        assert isinstance(value, dict)
        self[self._key_debug_options] = value

    @property
    def runtime_tmpdir(self):
        return self.setdefault(self._key_runtime_tmpdir, "")

    @runtime_tmpdir.setter
    def runtime_tmpdir(self, value):
        assert isinstance(value, str)
        self[self._key_runtime_tmpdir] = value

    @property
    def prioritize_venv(self):
        return self.setdefault(self._key_prioritize_venv, False)

    @prioritize_venv.setter
    def prioritize_venv(self, value):
        assert isinstance(value, bool)
        self[self._key_prioritize_venv] = value

    @property
    def encryption_key(self):
        return self.setdefault(self._key_encryption_key, "")

    @encryption_key.setter
    def encryption_key(self, value):
        assert isinstance(value, str)
        self[self._key_encryption_key] = value

    @property
    def uac_admin(self):
        return self.setdefault(self._key_uac_admin, False)

    @uac_admin.setter
    def uac_admin(self, value):
        assert isinstance(value, bool)
        self[self._key_uac_admin] = value

    @property
    def open_dist_folder(self):
        return self.setdefault(self._key_open_dist_folder, False)

    @open_dist_folder.setter
    def open_dist_folder(self, value):
        assert isinstance(value, bool)
        self[self._key_open_dist_folder] = value

    @property
    def delete_spec_file(self):
        return self.setdefault(self._key_delete_spec_file, False)

    @delete_spec_file.setter
    def delete_spec_file(self, value):
        assert isinstance(value, bool)
        self[self._key_delete_spec_file] = value

    @property
    def delete_working_dir(self):
        return self.setdefault(self._key_delete_working_dir, False)

    @delete_working_dir.setter
    def delete_working_dir(self, value):
        assert isinstance(value, bool)
        self[self._key_delete_working_dir] = value

    @property
    def hidden_imports(self):
        return self.setdefault(self._key_hidden_imports, list())

    @hidden_imports.setter
    def hidden_imports(self, value):
        assert isinstance(value, list)
        self[self._key_hidden_imports] = value

    @property
    def exclude_modules(self):
        return self.setdefault(self._key_exclude_modules, list())

    @exclude_modules.setter
    def exclude_modules(self, value):
        assert isinstance(value, list)
        self[self._key_exclude_modules] = value


class PyinstallerToolConfig(AbstractConfig):
    _key_window_size = "window_size"
    _key_impcheck_winsize = "impcheck_winsize"
    _key_envch_winsize = "envch_winsize"
    _key_current = "current"
    _key_multi_cfg = "multicfg"

    CONFIGFILE = "pyinstaller_assistant.json"

    def __init__(self):
        super().__init__(self.CONFIGFILE)

    def checkout_cfg(self, name: str) -> bool:
        if name not in self.multicfg:
            return False
        self.curconfig.clear()
        self.curconfig.update(self.multicfg[name])
        return True

    def store_curcfg(self, name: str) -> bool:
        if not name or not isinstance(name, str):
            return False
        new_settings_dict = dict()
        new_settings_dict.update(self.curconfig)
        self.multicfg[name] = deepcopy(new_settings_dict)
        return True

    @property
    def pypaths(self):
        shared_pypaths = get_shared_pypaths()
        if shared_pypaths is None:
            return list()
        assert isinstance(shared_pypaths, list)
        return shared_pypaths

    @property
    def multicfg(self) -> dict:
        if self._key_multi_cfg not in self:
            self[self._key_multi_cfg] = dict()
        return self[self._key_multi_cfg]

    @property
    def curconfig(self) -> PyiConfigure:
        if self._key_current not in self:
            self[self._key_current] = PyiConfigure()
        elif not isinstance(self[self._key_current], PyiConfigure):
            new_pyi_configure = PyiConfigure()
            new_pyi_configure.update(self[self._key_current])
            self[self._key_current] = new_pyi_configure
        return self[self._key_current]

    @property
    def window_size(self):
        if self._key_window_size not in self:
            self[self._key_window_size] = 960, 600
        return self[self._key_window_size]

    @window_size.setter
    def window_size(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_window_size] = value

    @property
    def envch_winsize(self):
        if self._key_envch_winsize not in self:
            self[self._key_envch_winsize] = 430, 200
        return self[self._key_envch_winsize]

    @envch_winsize.setter
    def envch_winsize(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_envch_winsize] = value

    @property
    def impcheck_winsize(self):
        if self._key_impcheck_winsize not in self:
            self[self._key_impcheck_winsize] = 960, 600
        return self[self._key_impcheck_winsize]

    @impcheck_winsize.setter
    def impcheck_winsize(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_impcheck_winsize] = value



# coding: utf-8

from typing import Sequence

from com import *

from .abstract_config import AbstractConfig


class MainEntranceConfig(AbstractConfig):
    _key_app_style = "app_style"
    _key_window_size = "window_size"

    CONFIGFILE = "main_entrance.json"

    def __init__(self):
        super().__init__(self.CONFIGFILE)

    @property
    def app_style(self):
        return self.setdefault(self._key_app_style, 2)

    @app_style.setter
    def app_style(self, value):
        assert isinstance(value, AppStyle)
        self[self._key_app_style] = value

    @property
    def window_size(self):
        if self._key_window_size not in self:
            self[self._key_window_size] = 260, 320
        return self[self._key_window_size]

    @window_size.setter
    def window_size(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_window_size] = value



# coding: utf-8

from typing import Sequence

from fastpip import index_urls

from .abstract_config import AbstractConfig
from .package_manager import get_shared_pypaths


class IndexManagerConfig(AbstractConfig):
    _key_index_urls = "index_urls"
    _key_window_size = "window_size"

    CONFIGFILE = "indexurl_manager.json"

    def __init__(self):
        super().__init__(self.CONFIGFILE)

    @property
    def index_urls(self):
        if self._key_index_urls not in self:
            self[self._key_index_urls] = index_urls.copy()
        return self[self._key_index_urls]

    @index_urls.setter
    def index_urls(self, value: dict):
        assert isinstance(value, dict)
        self[self._key_index_urls] = value

    @property
    def cur_pypaths(self):
        shared_pypaths = get_shared_pypaths()
        if shared_pypaths is None:
            return list()
        assert isinstance(shared_pypaths, list)
        return shared_pypaths

    @property
    def window_size(self):
        if self._key_window_size not in self:
            self[self._key_window_size] = 1000, 500
        return self[self._key_window_size]

    @window_size.setter
    def window_size(self, value):
        assert isinstance(value, Sequence)
        assert len(value) == 2
        assert isinstance(value[0], int) and isinstance(value[1], int)
        self[self._key_window_size] = value



# coding: utf-8

__doc__ = """包含 pyinstaller 工具相关的类或函数。"""

import os
from subprocess import *

from __info__ import *
from PyQt5.QtCore import *
from settings import *

from utils.main import get_cmd_out


class PyiTool(QObject):
    STARTUP = STARTUPINFO()
    STARTUP.dwFlags = STARTF_USESHOWWINDOW
    STARTUP.wShowWindow = SW_HIDE
    stdout = pyqtSignal(str)
    run_time = pyqtSignal(int)
    completed = pyqtSignal(int)

    def __init__(self, py_path="", cwd=os.getcwd()):
        super().__init__()
        self.__pypath = None
        self.__cwd = None
        self.__process = None
        self.__commands = None
        self.initialize(py_path, cwd)
        self.cumulative = -200
        self.__qtimer = QTimer()
        self.__qtimer.timeout.connect(self.__time)
        self.run_time.connect(self.__timer_control)
        self.__log_level = "INFO"

    @property
    def cwd(self):
        return self.__cwd

    @cwd.setter
    def cwd(self, path):
        if os.path.isdir(path):
            self.__cwd = path

    @property
    def pyi_path(self):
        """返回给出的 Python 路径中的 Pyinstaller 可执行文件路径。"""
        pyi_exec_path = os.path.join(self.__pypath, "Scripts", "pyinstaller.exe")
        if not os.path.isfile(pyi_exec_path):
            return ""
        return pyi_exec_path

    @property
    def pyi_ready(self):
        """给出的 Python 环境中安装了 Pyinstaller 返回 True，否则返回 False。"""
        return bool(self.pyi_path)

    def initialize(self, py_path, cwd):
        # 信任传入的 py_path
        self.__pypath = py_path
        self.__cwd = cwd if cwd else None
        self.__process = None
        self.__commands = [self.pyi_path]

    def __time(self):
        if self.cumulative > 10000:
            self.cumulative = 0
        self.cumulative += 10

    def __timer_control(self, code):
        if code:
            self.__qtimer.start(10)
        else:
            self.__qtimer.stop()
            self.cumulative = -200

    def __line_division_emit(self):
        while self.__process.poll() is None:
            try:
                line = self.__process.stdout.readline()
                if line is not None:
                    line = line.strip(os.linesep)
                    if line:
                        self.stdout.emit(line)
            except Exception as e:
                self.stdout.emit(f"[{NAME}] 信息流读取异常(不影响打包)：\n    {e}")
        self.completed.emit(self.__process.wait())

    def __time_division_emit(self):
        self.run_time.emit(1)
        lines = list()
        while self.__process.poll() is None:
            try:
                line = self.__process.stdout.readline()
                if line is not None:
                    line = line.strip(os.linesep)
                    if line:
                        lines.append(line)
                if self.cumulative > 100:
                    self.stdout.emit("\n".join(lines))
                    lines.clear()
                    self.cumulative = 0
            except Exception as e:
                self.stdout.emit(f"[{NAME}] 信息流读取异常(不影响打包)：\n    {e}")
        if lines:
            self.stdout.emit("\n".join(lines))
        self.run_time.emit(0)
        self.completed.emit(self.__process.wait())

    def execute_cmd(self):
        """执行命令并读取输出流，通过信号发射字符串、返回码更新主界面面板。"""
        self.__process = Popen(
            self.__commands,
            stdin=PIPE,
            stdout=PIPE,
            stderr=STDOUT,
            startupinfo=self.STARTUP,
            text=True,
            cwd=self.__cwd,
        )
        if self.pyi_ready and self.__process:
            if self.__log_level == "TRACE":
                self.__time_division_emit()
            else:
                self.__line_division_emit()
        else:
            if not self.pyi_ready:
                self.stdout.emit("当前环境中找不到 pyinstaller.exe。")
            if self.__process is None:
                self.stdout.emit("创建打包进程失败，无法完成程序打包。")
            self.completed.emit(-1)

    @staticmethod
    def __generate_infofile(verinfo_dict):
        version_info = """# coding: utf-8
VSVersionInfo(
ffi=FixedFileInfo(
filevers=$filevers$,
prodvers=$prodvers$,
mask=0x3F,
flags=0x0,
OS=0x40004,
fileType=0x1,
subtype=0x0,
date=(0, 0),
),
kids=[StringFileInfo(
[StringTable(
"080404b0",
[
StringStruct("CompanyName", "$CompanyName$"),
StringStruct("FileDescription", "$FileDescription$"),
StringStruct("FileVersion", "$FileVersion$"),
StringStruct("LegalCopyright", "$LegalCopyright$"),
StringStruct("OriginalFilename", "$OriginalFilename$"),
StringStruct("ProductName", "$ProductName$"),
StringStruct("ProductVersion", "$ProductVersion$"),
StringStruct("LegalTrademarks", "$LegalTrademarks$"),
],)]),
VarFileInfo([VarStruct("Translation", [2052, 1200])]),
],
)
"""
        file_path = os.path.join(config_root, "VERSOIN_INFO")
        for key, val in verinfo_dict.items():
            version_info = version_info.replace(key, val)
        try:
            with open(file_path, "wt", encoding="utf-8") as f:
                f.write(version_info)
        except Exception:
            file_path = ""
        return file_path

    def pyi_info(self):
        if self.pyi_ready:
            return get_cmd_out(self.pyi_path, "-v")
        return "0.0"

    def prepare_cmds(self, commands: PyiConfigure):
        """从 commands 添加 PyInstaller 命令选项。"""
        self.__log_level = commands.log_level
        if commands.onedir_bundle:
            self.__commands.append("-D")
        else:
            self.__commands.append("-F")
        if commands.spec_dir:
            self.__commands.extend(("--specpath", commands.spec_dir))
        if commands.bundle_spec_name:
            self.__commands.extend(("-n", commands.bundle_spec_name))
        for data in commands.other_datas:
            self.__commands.extend(("--add-data", rf"{data[0]};{data[1]}"))
        for module_path in commands.module_paths:
            self.__commands.extend(("-p", module_path))
        if commands.encryption_key:
            self.__commands.extend(("--key", commands.encryption_key))
        debug_options = commands.debug_options
        for option in debug_options:
            if debug_options[option]:
                self.__commands.extend(("--debug", option))
        if commands.donot_use_upx:
            self.__commands.append("--noupx")
        for binary in commands.upx_excludes:
            self.__commands.extend(("--upx-exclude", binary.lower()))
        if commands.provide_console:
            self.__commands.append("-c")
        else:
            self.__commands.append("-w")
        if commands.upx_dir:
            self.__commands.extend(("--upx-dir", commands.upx_dir))
        if commands.icon_path:
            self.__commands.extend(("-i", commands.icon_path))
        if commands.distribution_dir:
            self.__commands.extend(("--distpath", commands.distribution_dir))
        if commands.working_dir:
            self.__commands.extend(("--workpath", commands.working_dir))
        info_path = self.__generate_infofile(commands.version_info)
        if commands.add_verfile and info_path:
            self.__commands.extend(("--version-file", info_path))
        if commands.runtime_tmpdir:
            self.__commands.extend(("--runtime-tmpdir", commands.runtime_tmpdir))
        if commands.no_confirm:
            self.__commands.append("-y")
        if commands.clean_building:
            self.__commands.append("--clean")
        for imp in commands.hidden_imports:
            self.__commands.extend(("--hidden-import", imp))
        for mod in commands.exclude_modules:
            self.__commands.extend(("--exclude-module", mod))
        if commands.uac_admin:
            self.__commands.append("--uac-admin")
        self.__commands.extend(("--log-level", self.__log_level))
        self.__commands.append(commands.script_path)
